# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CREATE_SIGNED_DISTANCE_VOLUME_METADATA = Metadata(
    id="4d61fcbc40072b03f96c88c490ff99bbafba1379.workbench",
    name="create-signed-distance-volume",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CreateSignedDistanceVolumeRoiOutParamsDictNoTag = typing.TypedDict('_CreateSignedDistanceVolumeRoiOutParamsDictNoTag', {
    "roi-vol": str,
})
CreateSignedDistanceVolumeRoiOutParamsDictTagged = typing.TypedDict('CreateSignedDistanceVolumeRoiOutParamsDictTagged', {
    "@type": typing.Literal["roi-out"],
    "roi-vol": str,
})
CreateSignedDistanceVolumeRoiOutParamsDict = _CreateSignedDistanceVolumeRoiOutParamsDictNoTag | CreateSignedDistanceVolumeRoiOutParamsDictTagged


_CreateSignedDistanceVolumeParamsDictNoTag = typing.TypedDict('_CreateSignedDistanceVolumeParamsDictNoTag', {
    "surface": InputPathType,
    "refspace": str,
    "outvol": str,
    "roi-out": typing.NotRequired[CreateSignedDistanceVolumeRoiOutParamsDict | None],
    "method": typing.NotRequired[str | None],
    "num": typing.NotRequired[int | None],
    "dist": typing.NotRequired[float | None],
    "dist": typing.NotRequired[float | None],
    "value": typing.NotRequired[float | None],
})
CreateSignedDistanceVolumeParamsDictTagged = typing.TypedDict('CreateSignedDistanceVolumeParamsDictTagged', {
    "@type": typing.Literal["workbench/create-signed-distance-volume"],
    "surface": InputPathType,
    "refspace": str,
    "outvol": str,
    "roi-out": typing.NotRequired[CreateSignedDistanceVolumeRoiOutParamsDict | None],
    "method": typing.NotRequired[str | None],
    "num": typing.NotRequired[int | None],
    "dist": typing.NotRequired[float | None],
    "dist": typing.NotRequired[float | None],
    "value": typing.NotRequired[float | None],
})
CreateSignedDistanceVolumeParamsDict = _CreateSignedDistanceVolumeParamsDictNoTag | CreateSignedDistanceVolumeParamsDictTagged


class CreateSignedDistanceVolumeRoiOutOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CreateSignedDistanceVolumeRoiOutParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    roi_vol: OutputPathType
    """the output roi volume"""


def create_signed_distance_volume_roi_out(
    roi_vol: str,
) -> CreateSignedDistanceVolumeRoiOutParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_vol: the output roi volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi-out",
        "roi-vol": roi_vol,
    }
    return params


def create_signed_distance_volume_roi_out_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CreateSignedDistanceVolumeRoiOutParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-vol", None) is None:
        raise StyxValidationError("`roi-vol` must not be None")
    if not isinstance(params["roi-vol"], str):
        raise StyxValidationError(f'`roi-vol` has the wrong type: Received `{type(params.get("roi-vol", None))}` expected `str`')


def create_signed_distance_volume_roi_out_cargs(
    params: CreateSignedDistanceVolumeRoiOutParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi-out",
        params.get("roi-vol", None)
    ])
    return cargs


def create_signed_distance_volume_roi_out_outputs(
    params: CreateSignedDistanceVolumeRoiOutParamsDict,
    execution: Execution,
) -> CreateSignedDistanceVolumeRoiOutOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CreateSignedDistanceVolumeRoiOutOutputs(
        root=execution.output_file("."),
        roi_vol=execution.output_file(params.get("roi-vol", None)),
    )
    return ret


class CreateSignedDistanceVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CreateSignedDistanceVolumeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outvol: OutputPathType
    """the output volume"""
    roi_out: CreateSignedDistanceVolumeRoiOutOutputs | None
    """Outputs from `create_signed_distance_volume_roi_out_outputs`."""


def create_signed_distance_volume_params(
    surface: InputPathType,
    refspace: str,
    outvol: str,
    roi_out: CreateSignedDistanceVolumeRoiOutParamsDict | None = None,
    method: str | None = None,
    num: int | None = None,
    dist: float | None = None,
    dist_: float | None = None,
    value: float | None = None,
) -> CreateSignedDistanceVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the input surface.
        refspace: a volume in the desired output space (dims, spacing, origin).
        outvol: the output volume.
        roi_out: output an roi volume of where the output has a computed value.
        method: winding method for point inside surface test\
            \
            name of the method (default EVEN_ODD).
        num: voxel neighborhood for approximate calculation\
            \
            size of neighborhood cube measured from center to face, in voxels\
            (default 2 = 5x5x5).
        dist: specify distance for approximate output\
            \
            distance in mm (default 20).
        dist_: specify distance for exact output\
            \
            distance in mm (default 5).
        value: specify a value to put in all voxels that don't get assigned a\
            distance\
            \
            value to fill with (default 0).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/create-signed-distance-volume",
        "surface": surface,
        "refspace": refspace,
        "outvol": outvol,
    }
    if roi_out is not None:
        params["roi-out"] = roi_out
    if method is not None:
        params["method"] = method
    if num is not None:
        params["num"] = num
    if dist is not None:
        params["dist"] = dist
    if dist_ is not None:
        params["dist"] = dist_
    if value is not None:
        params["value"] = value
    return params


def create_signed_distance_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CreateSignedDistanceVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("refspace", None) is None:
        raise StyxValidationError("`refspace` must not be None")
    if not isinstance(params["refspace"], str):
        raise StyxValidationError(f'`refspace` has the wrong type: Received `{type(params.get("refspace", None))}` expected `str`')
    if params.get("outvol", None) is None:
        raise StyxValidationError("`outvol` must not be None")
    if not isinstance(params["outvol"], str):
        raise StyxValidationError(f'`outvol` has the wrong type: Received `{type(params.get("outvol", None))}` expected `str`')
    if params.get("roi-out", None) is not None:
        create_signed_distance_volume_roi_out_validate(params["roi-out"])
    if params.get("method", None) is not None:
        if not isinstance(params["method"], str):
            raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str | None`')
    if params.get("num", None) is not None:
        if not isinstance(params["num"], int):
            raise StyxValidationError(f'`num` has the wrong type: Received `{type(params.get("num", None))}` expected `int | None`')
    if params.get("dist", None) is not None:
        if not isinstance(params["dist"], (float, int)):
            raise StyxValidationError(f'`dist` has the wrong type: Received `{type(params.get("dist", None))}` expected `float | None`')
    if params.get("dist", None) is not None:
        if not isinstance(params["dist"], (float, int)):
            raise StyxValidationError(f'`dist` has the wrong type: Received `{type(params.get("dist", None))}` expected `float | None`')
    if params.get("value", None) is not None:
        if not isinstance(params["value"], (float, int)):
            raise StyxValidationError(f'`value` has the wrong type: Received `{type(params.get("value", None))}` expected `float | None`')


def create_signed_distance_volume_cargs(
    params: CreateSignedDistanceVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-create-signed-distance-volume"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(params.get("refspace", None))
    cargs.append(params.get("outvol", None))
    if params.get("roi-out", None) is not None:
        cargs.extend(create_signed_distance_volume_roi_out_cargs(params.get("roi-out", None), execution))
    if params.get("method", None) is not None:
        cargs.extend([
            "-winding",
            params.get("method", None)
        ])
    if params.get("num", None) is not None:
        cargs.extend([
            "-approx-neighborhood",
            str(params.get("num", None))
        ])
    if params.get("dist", None) is not None:
        cargs.extend([
            "-approx-limit",
            str(params.get("dist", None))
        ])
    if params.get("dist", None) is not None:
        cargs.extend([
            "-exact-limit",
            str(params.get("dist", None))
        ])
    if params.get("value", None) is not None:
        cargs.extend([
            "-fill-value",
            str(params.get("value", None))
        ])
    return cargs


def create_signed_distance_volume_outputs(
    params: CreateSignedDistanceVolumeParamsDict,
    execution: Execution,
) -> CreateSignedDistanceVolumeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CreateSignedDistanceVolumeOutputs(
        root=execution.output_file("."),
        outvol=execution.output_file(params.get("outvol", None)),
        roi_out=create_signed_distance_volume_roi_out_outputs(params.get("roi-out"), execution) if params.get("roi-out") else None,
    )
    return ret


def create_signed_distance_volume_execute(
    params: CreateSignedDistanceVolumeParamsDict,
    runner: Runner | None = None,
) -> CreateSignedDistanceVolumeOutputs:
    """
    CREATE SIGNED DISTANCE VOLUME FROM SURFACE.
    
    Computes the signed distance function of the surface. Exact distance is
    calculated by finding the closest point on any surface triangle to the
    center of the voxel. Approximate distance is calculated starting with these
    distances, using dijkstra's method with a neighborhood of voxels. Specifying
    too small of an exact distance may produce unexpected results. Valid
    specifiers for winding methods are as follows:
    
    EVEN_ODD (default)
    NEGATIVE
    NONZERO
    NORMALS
    
    The NORMALS method uses the normals of triangles and edges, or the closest
    triangle hit by a ray from the point. This method may be slightly faster,
    but is only reliable for a closed surface that does not cross through
    itself. All other methods count entry (positive) and exit (negative)
    crossings of a vertical ray from the point, then counts as inside if the
    total is odd, negative, or nonzero, respectively.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateSignedDistanceVolumeOutputs`).
    """
    create_signed_distance_volume_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CREATE_SIGNED_DISTANCE_VOLUME_METADATA)
    params = execution.params(params)
    cargs = create_signed_distance_volume_cargs(params, execution)
    ret = create_signed_distance_volume_outputs(params, execution)
    execution.run(cargs)
    return ret


def create_signed_distance_volume(
    surface: InputPathType,
    refspace: str,
    outvol: str,
    roi_out: CreateSignedDistanceVolumeRoiOutParamsDict | None = None,
    method: str | None = None,
    num: int | None = None,
    dist: float | None = None,
    dist_: float | None = None,
    value: float | None = None,
    runner: Runner | None = None,
) -> CreateSignedDistanceVolumeOutputs:
    """
    CREATE SIGNED DISTANCE VOLUME FROM SURFACE.
    
    Computes the signed distance function of the surface. Exact distance is
    calculated by finding the closest point on any surface triangle to the
    center of the voxel. Approximate distance is calculated starting with these
    distances, using dijkstra's method with a neighborhood of voxels. Specifying
    too small of an exact distance may produce unexpected results. Valid
    specifiers for winding methods are as follows:
    
    EVEN_ODD (default)
    NEGATIVE
    NONZERO
    NORMALS
    
    The NORMALS method uses the normals of triangles and edges, or the closest
    triangle hit by a ray from the point. This method may be slightly faster,
    but is only reliable for a closed surface that does not cross through
    itself. All other methods count entry (positive) and exit (negative)
    crossings of a vertical ray from the point, then counts as inside if the
    total is odd, negative, or nonzero, respectively.
    
    Args:
        surface: the input surface.
        refspace: a volume in the desired output space (dims, spacing, origin).
        outvol: the output volume.
        roi_out: output an roi volume of where the output has a computed value.
        method: winding method for point inside surface test\
            \
            name of the method (default EVEN_ODD).
        num: voxel neighborhood for approximate calculation\
            \
            size of neighborhood cube measured from center to face, in voxels\
            (default 2 = 5x5x5).
        dist: specify distance for approximate output\
            \
            distance in mm (default 20).
        dist_: specify distance for exact output\
            \
            distance in mm (default 5).
        value: specify a value to put in all voxels that don't get assigned a\
            distance\
            \
            value to fill with (default 0).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CreateSignedDistanceVolumeOutputs`).
    """
    params = create_signed_distance_volume_params(
        surface=surface,
        refspace=refspace,
        outvol=outvol,
        roi_out=roi_out,
        method=method,
        num=num,
        dist=dist,
        dist_=dist_,
        value=value,
    )
    return create_signed_distance_volume_execute(params, runner)


__all__ = [
    "CREATE_SIGNED_DISTANCE_VOLUME_METADATA",
    "CreateSignedDistanceVolumeOutputs",
    "CreateSignedDistanceVolumeParamsDict",
    "CreateSignedDistanceVolumeParamsDictTagged",
    "CreateSignedDistanceVolumeRoiOutOutputs",
    "CreateSignedDistanceVolumeRoiOutParamsDict",
    "CreateSignedDistanceVolumeRoiOutParamsDictTagged",
    "create_signed_distance_volume",
    "create_signed_distance_volume_execute",
    "create_signed_distance_volume_params",
    "create_signed_distance_volume_roi_out",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_MERGE_METADATA = Metadata(
    id="791909e8270318bd504b97ada9b61a7c8e2898d1.workbench",
    name="label-merge",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelMergeUpToParamsDictNoTag = typing.TypedDict('_LabelMergeUpToParamsDictNoTag', {
    "last-column": str,
    "reverse": bool,
})
LabelMergeUpToParamsDictTagged = typing.TypedDict('LabelMergeUpToParamsDictTagged', {
    "@type": typing.Literal["up-to"],
    "last-column": str,
    "reverse": bool,
})
LabelMergeUpToParamsDict = _LabelMergeUpToParamsDictNoTag | LabelMergeUpToParamsDictTagged


_LabelMergeColumnParamsDictNoTag = typing.TypedDict('_LabelMergeColumnParamsDictNoTag', {
    "column": str,
    "up-to": typing.NotRequired[LabelMergeUpToParamsDict | None],
})
LabelMergeColumnParamsDictTagged = typing.TypedDict('LabelMergeColumnParamsDictTagged', {
    "@type": typing.Literal["column"],
    "column": str,
    "up-to": typing.NotRequired[LabelMergeUpToParamsDict | None],
})
LabelMergeColumnParamsDict = _LabelMergeColumnParamsDictNoTag | LabelMergeColumnParamsDictTagged


_LabelMergeLabelParamsDictNoTag = typing.TypedDict('_LabelMergeLabelParamsDictNoTag', {
    "label-in": InputPathType,
    "column": typing.NotRequired[list[LabelMergeColumnParamsDict] | None],
})
LabelMergeLabelParamsDictTagged = typing.TypedDict('LabelMergeLabelParamsDictTagged', {
    "@type": typing.Literal["label"],
    "label-in": InputPathType,
    "column": typing.NotRequired[list[LabelMergeColumnParamsDict] | None],
})
LabelMergeLabelParamsDict = _LabelMergeLabelParamsDictNoTag | LabelMergeLabelParamsDictTagged


_LabelMergeParamsDictNoTag = typing.TypedDict('_LabelMergeParamsDictNoTag', {
    "label-out": str,
    "label": typing.NotRequired[list[LabelMergeLabelParamsDict] | None],
})
LabelMergeParamsDictTagged = typing.TypedDict('LabelMergeParamsDictTagged', {
    "@type": typing.Literal["workbench/label-merge"],
    "label-out": str,
    "label": typing.NotRequired[list[LabelMergeLabelParamsDict] | None],
})
LabelMergeParamsDict = _LabelMergeParamsDictNoTag | LabelMergeParamsDictTagged


def label_merge_up_to(
    last_column: str,
    reverse: bool = False,
) -> LabelMergeUpToParamsDictTagged:
    """
    Build parameters.
    
    Args:
        last_column: the number or name of the last column to include.
        reverse: use the range in reverse order.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "up-to",
        "last-column": last_column,
        "reverse": reverse,
    }
    return params


def label_merge_up_to_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelMergeUpToParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("last-column", None) is None:
        raise StyxValidationError("`last-column` must not be None")
    if not isinstance(params["last-column"], str):
        raise StyxValidationError(f'`last-column` has the wrong type: Received `{type(params.get("last-column", None))}` expected `str`')
    if params.get("reverse", False) is None:
        raise StyxValidationError("`reverse` must not be None")
    if not isinstance(params["reverse"], bool):
        raise StyxValidationError(f'`reverse` has the wrong type: Received `{type(params.get("reverse", False))}` expected `bool`')


def label_merge_up_to_cargs(
    params: LabelMergeUpToParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-up-to",
        params.get("last-column", None)
    ])
    if params.get("reverse", False):
        cargs.append("-reverse")
    return cargs


def label_merge_column(
    column: str,
    up_to: LabelMergeUpToParamsDict | None = None,
) -> LabelMergeColumnParamsDictTagged:
    """
    Build parameters.
    
    Args:
        column: the column number or name.
        up_to: use an inclusive range of columns.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "column",
        "column": column,
    }
    if up_to is not None:
        params["up-to"] = up_to
    return params


def label_merge_column_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelMergeColumnParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("column", None) is None:
        raise StyxValidationError("`column` must not be None")
    if not isinstance(params["column"], str):
        raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str`')
    if params.get("up-to", None) is not None:
        label_merge_up_to_validate(params["up-to"])


def label_merge_column_cargs(
    params: LabelMergeColumnParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-column",
        params.get("column", None),
        *(label_merge_up_to_cargs(params.get("up-to", None), execution) if (params.get("up-to", None) is not None) else [])
    ])
    return cargs


def label_merge_label(
    label_in: InputPathType,
    column: list[LabelMergeColumnParamsDict] | None = None,
) -> LabelMergeLabelParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_in: a label file to use columns from.
        column: select a single column to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "label",
        "label-in": label_in,
    }
    if column is not None:
        params["column"] = column
    return params


def label_merge_label_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelMergeLabelParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-in", None) is None:
        raise StyxValidationError("`label-in` must not be None")
    if not isinstance(params["label-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-in` has the wrong type: Received `{type(params.get("label-in", None))}` expected `InputPathType`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], list):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `list[LabelMergeColumnParamsDict] | None`')
        for e in params["column"]:
            label_merge_column_validate(e)


def label_merge_label_cargs(
    params: LabelMergeLabelParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-label",
        execution.input_file(params.get("label-in", None)),
        *([a for c in [label_merge_column_cargs(s, execution) for s in params.get("column", None)] for a in c] if (params.get("column", None) is not None) else [])
    ])
    return cargs


class LabelMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelMergeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label"""


def label_merge_params(
    label_out: str,
    label: list[LabelMergeLabelParamsDict] | None = None,
) -> LabelMergeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_out: the output label.
        label: specify an input label.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-merge",
        "label-out": label_out,
    }
    if label is not None:
        params["label"] = label
    return params


def label_merge_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelMergeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')
    if params.get("label", None) is not None:
        if not isinstance(params["label"], list):
            raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `list[LabelMergeLabelParamsDict] | None`')
        for e in params["label"]:
            label_merge_label_validate(e)


def label_merge_cargs(
    params: LabelMergeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-merge"
    ])
    cargs.append(params.get("label-out", None))
    if params.get("label", None) is not None:
        cargs.extend([a for c in [label_merge_label_cargs(s, execution) for s in params.get("label", None)] for a in c])
    return cargs


def label_merge_outputs(
    params: LabelMergeParamsDict,
    execution: Execution,
) -> LabelMergeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelMergeOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
    )
    return ret


def label_merge_execute(
    params: LabelMergeParamsDict,
    runner: Runner | None = None,
) -> LabelMergeOutputs:
    """
    MERGE LABEL FILES INTO A NEW FILE.
    
    Takes one or more label files and constructs a new label file by
    concatenating columns from them. The input files must have the same number
    of vertices and the same structure.
    
    Example: wb_command -label-merge out.label.gii -label first.label.gii
    -column 1 -label second.label.gii
    
    This example would take the first column from first.label.gii and all
    subvolumes from second.label.gii, and write these to out.label.gii.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelMergeOutputs`).
    """
    label_merge_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_MERGE_METADATA)
    params = execution.params(params)
    cargs = label_merge_cargs(params, execution)
    ret = label_merge_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_merge(
    label_out: str,
    label: list[LabelMergeLabelParamsDict] | None = None,
    runner: Runner | None = None,
) -> LabelMergeOutputs:
    """
    MERGE LABEL FILES INTO A NEW FILE.
    
    Takes one or more label files and constructs a new label file by
    concatenating columns from them. The input files must have the same number
    of vertices and the same structure.
    
    Example: wb_command -label-merge out.label.gii -label first.label.gii
    -column 1 -label second.label.gii
    
    This example would take the first column from first.label.gii and all
    subvolumes from second.label.gii, and write these to out.label.gii.
    
    Args:
        label_out: the output label.
        label: specify an input label.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelMergeOutputs`).
    """
    params = label_merge_params(
        label_out=label_out,
        label=label,
    )
    return label_merge_execute(params, runner)


__all__ = [
    "LABEL_MERGE_METADATA",
    "LabelMergeColumnParamsDict",
    "LabelMergeColumnParamsDictTagged",
    "LabelMergeLabelParamsDict",
    "LabelMergeLabelParamsDictTagged",
    "LabelMergeOutputs",
    "LabelMergeParamsDict",
    "LabelMergeParamsDictTagged",
    "LabelMergeUpToParamsDict",
    "LabelMergeUpToParamsDictTagged",
    "label_merge",
    "label_merge_column",
    "label_merge_execute",
    "label_merge_label",
    "label_merge_params",
    "label_merge_up_to",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CONVERT_WARPFIELD_METADATA = Metadata(
    id="4f9d440378d5e5e478898e11b30a204baa1a8db9.workbench",
    name="convert-warpfield",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_ConvertWarpfieldFromWorldParamsDictNoTag = typing.TypedDict('_ConvertWarpfieldFromWorldParamsDictNoTag', {
    "input": str,
    "absolute": bool,
})
ConvertWarpfieldFromWorldParamsDictTagged = typing.TypedDict('ConvertWarpfieldFromWorldParamsDictTagged', {
    "@type": typing.Literal["from-world"],
    "input": str,
    "absolute": bool,
})
ConvertWarpfieldFromWorldParamsDict = _ConvertWarpfieldFromWorldParamsDictNoTag | ConvertWarpfieldFromWorldParamsDictTagged


_ConvertWarpfieldFromFnirtParamsDictNoTag = typing.TypedDict('_ConvertWarpfieldFromFnirtParamsDictNoTag', {
    "input": str,
    "source-volume": str,
    "absolute": bool,
})
ConvertWarpfieldFromFnirtParamsDictTagged = typing.TypedDict('ConvertWarpfieldFromFnirtParamsDictTagged', {
    "@type": typing.Literal["from-fnirt"],
    "input": str,
    "source-volume": str,
    "absolute": bool,
})
ConvertWarpfieldFromFnirtParamsDict = _ConvertWarpfieldFromFnirtParamsDictNoTag | ConvertWarpfieldFromFnirtParamsDictTagged


_ConvertWarpfieldToFnirtParamsDictNoTag = typing.TypedDict('_ConvertWarpfieldToFnirtParamsDictNoTag', {
    "output": str,
    "source-volume": str,
})
ConvertWarpfieldToFnirtParamsDictTagged = typing.TypedDict('ConvertWarpfieldToFnirtParamsDictTagged', {
    "@type": typing.Literal["to-fnirt"],
    "output": str,
    "source-volume": str,
})
ConvertWarpfieldToFnirtParamsDict = _ConvertWarpfieldToFnirtParamsDictNoTag | ConvertWarpfieldToFnirtParamsDictTagged


_ConvertWarpfieldParamsDictNoTag = typing.TypedDict('_ConvertWarpfieldParamsDictNoTag', {
    "from-world": typing.NotRequired[ConvertWarpfieldFromWorldParamsDict | None],
    "from-fnirt": typing.NotRequired[ConvertWarpfieldFromFnirtParamsDict | None],
    "to-fnirt": typing.NotRequired[list[ConvertWarpfieldToFnirtParamsDict] | None],
    "output": typing.NotRequired[str | None],
    "output": typing.NotRequired[str | None],
    "input": typing.NotRequired[str | None],
})
ConvertWarpfieldParamsDictTagged = typing.TypedDict('ConvertWarpfieldParamsDictTagged', {
    "@type": typing.Literal["workbench/convert-warpfield"],
    "from-world": typing.NotRequired[ConvertWarpfieldFromWorldParamsDict | None],
    "from-fnirt": typing.NotRequired[ConvertWarpfieldFromFnirtParamsDict | None],
    "to-fnirt": typing.NotRequired[list[ConvertWarpfieldToFnirtParamsDict] | None],
    "output": typing.NotRequired[str | None],
    "output": typing.NotRequired[str | None],
    "input": typing.NotRequired[str | None],
})
ConvertWarpfieldParamsDict = _ConvertWarpfieldParamsDictNoTag | ConvertWarpfieldParamsDictTagged


def convert_warpfield_from_world(
    input_: str,
    absolute: bool = False,
) -> ConvertWarpfieldFromWorldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input warpfield.
        absolute: warpfield was written in absolute convention, rather than\
            relative.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-world",
        "input": input_,
        "absolute": absolute,
    }
    return params


def convert_warpfield_from_world_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertWarpfieldFromWorldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], str):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `str`')
    if params.get("absolute", False) is None:
        raise StyxValidationError("`absolute` must not be None")
    if not isinstance(params["absolute"], bool):
        raise StyxValidationError(f'`absolute` has the wrong type: Received `{type(params.get("absolute", False))}` expected `bool`')


def convert_warpfield_from_world_cargs(
    params: ConvertWarpfieldFromWorldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-world",
        params.get("input", None)
    ])
    if params.get("absolute", False):
        cargs.append("-absolute")
    return cargs


def convert_warpfield_from_fnirt(
    input_: str,
    source_volume: str,
    absolute: bool = False,
) -> ConvertWarpfieldFromFnirtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: the input warpfield.
        source_volume: the source volume used when generating the input\
            warpfield.
        absolute: warpfield was written in absolute convention, rather than\
            relative.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-fnirt",
        "input": input_,
        "source-volume": source_volume,
        "absolute": absolute,
    }
    return params


def convert_warpfield_from_fnirt_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertWarpfieldFromFnirtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], str):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `str`')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')
    if params.get("absolute", False) is None:
        raise StyxValidationError("`absolute` must not be None")
    if not isinstance(params["absolute"], bool):
        raise StyxValidationError(f'`absolute` has the wrong type: Received `{type(params.get("absolute", False))}` expected `bool`')


def convert_warpfield_from_fnirt_cargs(
    params: ConvertWarpfieldFromFnirtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-fnirt",
        params.get("input", None),
        params.get("source-volume", None)
    ])
    if params.get("absolute", False):
        cargs.append("-absolute")
    return cargs


def convert_warpfield_to_fnirt(
    output: str,
    source_volume: str,
) -> ConvertWarpfieldToFnirtParamsDictTagged:
    """
    Build parameters.
    
    Args:
        output: output - the output warpfield.
        source_volume: the volume you want to apply the warpfield to.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "to-fnirt",
        "output": output,
        "source-volume": source_volume,
    }
    return params


def convert_warpfield_to_fnirt_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertWarpfieldToFnirtParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("output", None) is None:
        raise StyxValidationError("`output` must not be None")
    if not isinstance(params["output"], str):
        raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str`')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')


def convert_warpfield_to_fnirt_cargs(
    params: ConvertWarpfieldToFnirtParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-to-fnirt",
        params.get("output", None),
        params.get("source-volume", None)
    ])
    return cargs


class ConvertWarpfieldOutputs(typing.NamedTuple):
    """
    Output object returned when calling `ConvertWarpfieldParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def convert_warpfield_params(
    from_world: ConvertWarpfieldFromWorldParamsDict | None = None,
    from_fnirt: ConvertWarpfieldFromFnirtParamsDict | None = None,
    to_fnirt: list[ConvertWarpfieldToFnirtParamsDict] | None = None,
    output: str | None = None,
    output_: str | None = None,
    input_: str | None = None,
) -> ConvertWarpfieldParamsDictTagged:
    """
    Build parameters.
    
    Args:
        from_world: input is a NIFTI 'world' warpfield.
        from_fnirt: input is a fnirt warpfield.
        to_fnirt: write output as a fnirt warpfield.
        output: write output as an ITK warpfield\
            \
            output - the output warpfield.
        output_: write output as a NIFTI 'world' warpfield\
            \
            output - the output warpfield.
        input_: input is an ITK warpfield\
            \
            the input warpfield.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/convert-warpfield",
    }
    if from_world is not None:
        params["from-world"] = from_world
    if from_fnirt is not None:
        params["from-fnirt"] = from_fnirt
    if to_fnirt is not None:
        params["to-fnirt"] = to_fnirt
    if output is not None:
        params["output"] = output
    if output_ is not None:
        params["output"] = output_
    if input_ is not None:
        params["input"] = input_
    return params


def convert_warpfield_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `ConvertWarpfieldParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("from-world", None) is not None:
        convert_warpfield_from_world_validate(params["from-world"])
    if params.get("from-fnirt", None) is not None:
        convert_warpfield_from_fnirt_validate(params["from-fnirt"])
    if params.get("to-fnirt", None) is not None:
        if not isinstance(params["to-fnirt"], list):
            raise StyxValidationError(f'`to-fnirt` has the wrong type: Received `{type(params.get("to-fnirt", None))}` expected `list[ConvertWarpfieldToFnirtParamsDict] | None`')
        for e in params["to-fnirt"]:
            convert_warpfield_to_fnirt_validate(e)
    if params.get("output", None) is not None:
        if not isinstance(params["output"], str):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str | None`')
    if params.get("output", None) is not None:
        if not isinstance(params["output"], str):
            raise StyxValidationError(f'`output` has the wrong type: Received `{type(params.get("output", None))}` expected `str | None`')
    if params.get("input", None) is not None:
        if not isinstance(params["input"], str):
            raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `str | None`')


def convert_warpfield_cargs(
    params: ConvertWarpfieldParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-convert-warpfield"
    ])
    if params.get("from-world", None) is not None or params.get("from-fnirt", None) is not None or params.get("to-fnirt", None) is not None:
        cargs.extend([
            *(convert_warpfield_from_world_cargs(params.get("from-world", None), execution) if (params.get("from-world", None) is not None) else []),
            *(convert_warpfield_from_fnirt_cargs(params.get("from-fnirt", None), execution) if (params.get("from-fnirt", None) is not None) else []),
            *([a for c in [convert_warpfield_to_fnirt_cargs(s, execution) for s in params.get("to-fnirt", None)] for a in c] if (params.get("to-fnirt", None) is not None) else [])
        ])
    if params.get("output", None) is not None:
        cargs.extend([
            "-to-itk",
            params.get("output", None)
        ])
    if params.get("output", None) is not None:
        cargs.extend([
            "-to-world",
            params.get("output", None)
        ])
    if params.get("input", None) is not None:
        cargs.extend([
            "-from-itk",
            params.get("input", None)
        ])
    return cargs


def convert_warpfield_outputs(
    params: ConvertWarpfieldParamsDict,
    execution: Execution,
) -> ConvertWarpfieldOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = ConvertWarpfieldOutputs(
        root=execution.output_file("."),
    )
    return ret


def convert_warpfield_execute(
    params: ConvertWarpfieldParamsDict,
    runner: Runner | None = None,
) -> ConvertWarpfieldOutputs:
    """
    CONVERT A WARPFIELD BETWEEN CONVENTIONS.
    
    NIFTI world warpfields can be used directly on mm coordinates via sampling
    the three subvolumes at the coordinate and adding the sampled values to the
    coordinate vector. They use the NIFTI coordinate system, that is, X is left
    to right, Y is posterior to anterior, and Z is inferior to superior.
    
    NOTE: this command does not invert the warpfield, and to warp a surface, you
    must use the inverse of the warpfield that warps the corresponding volume.
    
    The ITK format is used by ANTS.
    
    You must specify exactly one -from option, but you may specify multiple -to
    options, and -to-fnirt may be specified more than once.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertWarpfieldOutputs`).
    """
    convert_warpfield_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CONVERT_WARPFIELD_METADATA)
    params = execution.params(params)
    cargs = convert_warpfield_cargs(params, execution)
    ret = convert_warpfield_outputs(params, execution)
    execution.run(cargs)
    return ret


def convert_warpfield(
    from_world: ConvertWarpfieldFromWorldParamsDict | None = None,
    from_fnirt: ConvertWarpfieldFromFnirtParamsDict | None = None,
    to_fnirt: list[ConvertWarpfieldToFnirtParamsDict] | None = None,
    output: str | None = None,
    output_: str | None = None,
    input_: str | None = None,
    runner: Runner | None = None,
) -> ConvertWarpfieldOutputs:
    """
    CONVERT A WARPFIELD BETWEEN CONVENTIONS.
    
    NIFTI world warpfields can be used directly on mm coordinates via sampling
    the three subvolumes at the coordinate and adding the sampled values to the
    coordinate vector. They use the NIFTI coordinate system, that is, X is left
    to right, Y is posterior to anterior, and Z is inferior to superior.
    
    NOTE: this command does not invert the warpfield, and to warp a surface, you
    must use the inverse of the warpfield that warps the corresponding volume.
    
    The ITK format is used by ANTS.
    
    You must specify exactly one -from option, but you may specify multiple -to
    options, and -to-fnirt may be specified more than once.
    
    Args:
        from_world: input is a NIFTI 'world' warpfield.
        from_fnirt: input is a fnirt warpfield.
        to_fnirt: write output as a fnirt warpfield.
        output: write output as an ITK warpfield\
            \
            output - the output warpfield.
        output_: write output as a NIFTI 'world' warpfield\
            \
            output - the output warpfield.
        input_: input is an ITK warpfield\
            \
            the input warpfield.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ConvertWarpfieldOutputs`).
    """
    params = convert_warpfield_params(
        from_world=from_world,
        from_fnirt=from_fnirt,
        to_fnirt=to_fnirt,
        output=output,
        output_=output_,
        input_=input_,
    )
    return convert_warpfield_execute(params, runner)


__all__ = [
    "CONVERT_WARPFIELD_METADATA",
    "ConvertWarpfieldFromFnirtParamsDict",
    "ConvertWarpfieldFromFnirtParamsDictTagged",
    "ConvertWarpfieldFromWorldParamsDict",
    "ConvertWarpfieldFromWorldParamsDictTagged",
    "ConvertWarpfieldOutputs",
    "ConvertWarpfieldParamsDict",
    "ConvertWarpfieldParamsDictTagged",
    "ConvertWarpfieldToFnirtParamsDict",
    "ConvertWarpfieldToFnirtParamsDictTagged",
    "convert_warpfield",
    "convert_warpfield_execute",
    "convert_warpfield_from_fnirt",
    "convert_warpfield_from_world",
    "convert_warpfield_params",
    "convert_warpfield_to_fnirt",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_DILATE_METADATA = Metadata(
    id="5f7bbed13558b8a40bfedd7f35e6b45da7a74ffa.workbench",
    name="metric-dilate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricDilateParamsDictNoTag = typing.TypedDict('_MetricDilateParamsDictNoTag', {
    "metric": InputPathType,
    "surface": InputPathType,
    "distance": float,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "exponent": typing.NotRequired[float | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "legacy-cutoff": bool,
    "linear": bool,
    "nearest": bool,
})
MetricDilateParamsDictTagged = typing.TypedDict('MetricDilateParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-dilate"],
    "metric": InputPathType,
    "surface": InputPathType,
    "distance": float,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "exponent": typing.NotRequired[float | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
    "legacy-cutoff": bool,
    "linear": bool,
    "nearest": bool,
})
MetricDilateParamsDict = _MetricDilateParamsDictNoTag | MetricDilateParamsDictTagged


class MetricDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricDilateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_dilate_params(
    metric: InputPathType,
    surface: InputPathType,
    distance: float,
    metric_out: str,
    area_metric: InputPathType | None = None,
    exponent: float | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    legacy_cutoff: bool = False,
    linear: bool = False,
    nearest: bool = False,
) -> MetricDilateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric to dilate.
        surface: the surface to compute on.
        distance: distance in mm to dilate.
        metric_out: the output metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        exponent: use a different exponent in the weighting function\
            \
            exponent 'n' to use in (area / (distance ^ n)) as the weighting\
            function (default 6).
        column: select a single column to dilate\
            \
            the column number or name.
        roi_metric: specify an roi of where there is data\
            \
            metric file, positive values denote vertices that have data.
        roi_metric_: specify an roi of vertices to overwrite, rather than\
            vertices with value zero\
            \
            metric file, positive values denote vertices to have their values\
            replaced.
        legacy_cutoff: use the v1.3.2 method of choosing how many vertices to\
            use when calulating the dilated value with weighted method.
        linear: fill in values with linear interpolation along strongest\
            gradient.
        nearest: use the nearest good value instead of a weighted average.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-dilate",
        "metric": metric,
        "surface": surface,
        "distance": distance,
        "metric-out": metric_out,
        "legacy-cutoff": legacy_cutoff,
        "linear": linear,
        "nearest": nearest,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    if exponent is not None:
        params["exponent"] = exponent
    if column is not None:
        params["column"] = column
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    if roi_metric_ is not None:
        params["roi-metric"] = roi_metric_
    return params


def metric_dilate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricDilateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("distance", None) is None:
        raise StyxValidationError("`distance` must not be None")
    if not isinstance(params["distance"], (float, int)):
        raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("exponent", None) is not None:
        if not isinstance(params["exponent"], (float, int)):
            raise StyxValidationError(f'`exponent` has the wrong type: Received `{type(params.get("exponent", None))}` expected `float | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')
    if params.get("legacy-cutoff", False) is None:
        raise StyxValidationError("`legacy-cutoff` must not be None")
    if not isinstance(params["legacy-cutoff"], bool):
        raise StyxValidationError(f'`legacy-cutoff` has the wrong type: Received `{type(params.get("legacy-cutoff", False))}` expected `bool`')
    if params.get("linear", False) is None:
        raise StyxValidationError("`linear` must not be None")
    if not isinstance(params["linear"], bool):
        raise StyxValidationError(f'`linear` has the wrong type: Received `{type(params.get("linear", False))}` expected `bool`')
    if params.get("nearest", False) is None:
        raise StyxValidationError("`nearest` must not be None")
    if not isinstance(params["nearest"], bool):
        raise StyxValidationError(f'`nearest` has the wrong type: Received `{type(params.get("nearest", False))}` expected `bool`')


def metric_dilate_cargs(
    params: MetricDilateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-dilate"
    ])
    cargs.append(execution.input_file(params.get("metric", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("exponent", None) is not None:
        cargs.extend([
            "-exponent",
            str(params.get("exponent", None))
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-data-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-bad-vertex-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    if params.get("legacy-cutoff", False):
        cargs.append("-legacy-cutoff")
    if params.get("linear", False):
        cargs.append("-linear")
    if params.get("nearest", False):
        cargs.append("-nearest")
    return cargs


def metric_dilate_outputs(
    params: MetricDilateParamsDict,
    execution: Execution,
) -> MetricDilateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricDilateOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_dilate_execute(
    params: MetricDilateParamsDict,
    runner: Runner | None = None,
) -> MetricDilateOutputs:
    """
    DILATE A METRIC FILE.
    
    For all metric vertices that are designated as bad, if they neighbor a
    non-bad vertex with data or are within the specified distance of such a
    vertex, replace the value with a distance-based weighted average of nearby
    non-bad vertices that have data, otherwise set the value to zero. No matter
    how small <distance> is, dilation will always use at least the immediate
    neighbor vertices. If -nearest is specified, it will use the value from the
    closest non-bad vertex with data within range instead of a weighted average.
    
    If -bad-vertex-roi is specified, all vertices with a positive ROI value are
    bad. If it is not specified, only vertices that have data, with a value of
    zero, are bad. If -data-roi is not specified, all vertices are assumed to
    have data.
    
    Note that the -corrected-areas option uses an approximate correction for the
    change in distances along a group average surface.
    
    To get the behavior of version 1.3.2 or earlier, use '-legacy-cutoff
    -exponent 2'.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricDilateOutputs`).
    """
    metric_dilate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_DILATE_METADATA)
    params = execution.params(params)
    cargs = metric_dilate_cargs(params, execution)
    ret = metric_dilate_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_dilate(
    metric: InputPathType,
    surface: InputPathType,
    distance: float,
    metric_out: str,
    area_metric: InputPathType | None = None,
    exponent: float | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    roi_metric_: InputPathType | None = None,
    legacy_cutoff: bool = False,
    linear: bool = False,
    nearest: bool = False,
    runner: Runner | None = None,
) -> MetricDilateOutputs:
    """
    DILATE A METRIC FILE.
    
    For all metric vertices that are designated as bad, if they neighbor a
    non-bad vertex with data or are within the specified distance of such a
    vertex, replace the value with a distance-based weighted average of nearby
    non-bad vertices that have data, otherwise set the value to zero. No matter
    how small <distance> is, dilation will always use at least the immediate
    neighbor vertices. If -nearest is specified, it will use the value from the
    closest non-bad vertex with data within range instead of a weighted average.
    
    If -bad-vertex-roi is specified, all vertices with a positive ROI value are
    bad. If it is not specified, only vertices that have data, with a value of
    zero, are bad. If -data-roi is not specified, all vertices are assumed to
    have data.
    
    Note that the -corrected-areas option uses an approximate correction for the
    change in distances along a group average surface.
    
    To get the behavior of version 1.3.2 or earlier, use '-legacy-cutoff
    -exponent 2'.
    
    Args:
        metric: the metric to dilate.
        surface: the surface to compute on.
        distance: distance in mm to dilate.
        metric_out: the output metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        exponent: use a different exponent in the weighting function\
            \
            exponent 'n' to use in (area / (distance ^ n)) as the weighting\
            function (default 6).
        column: select a single column to dilate\
            \
            the column number or name.
        roi_metric: specify an roi of where there is data\
            \
            metric file, positive values denote vertices that have data.
        roi_metric_: specify an roi of vertices to overwrite, rather than\
            vertices with value zero\
            \
            metric file, positive values denote vertices to have their values\
            replaced.
        legacy_cutoff: use the v1.3.2 method of choosing how many vertices to\
            use when calulating the dilated value with weighted method.
        linear: fill in values with linear interpolation along strongest\
            gradient.
        nearest: use the nearest good value instead of a weighted average.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricDilateOutputs`).
    """
    params = metric_dilate_params(
        metric=metric,
        surface=surface,
        distance=distance,
        metric_out=metric_out,
        area_metric=area_metric,
        exponent=exponent,
        column=column,
        roi_metric=roi_metric,
        roi_metric_=roi_metric_,
        legacy_cutoff=legacy_cutoff,
        linear=linear,
        nearest=nearest,
    )
    return metric_dilate_execute(params, runner)


__all__ = [
    "METRIC_DILATE_METADATA",
    "MetricDilateOutputs",
    "MetricDilateParamsDict",
    "MetricDilateParamsDictTagged",
    "metric_dilate",
    "metric_dilate_execute",
    "metric_dilate_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_WEIGHTED_STATS_METADATA = Metadata(
    id="4e95434c5f2fd06962a53dcd7657c6c2bbb93a7b.workbench",
    name="volume-weighted-stats",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeWeightedStatsWeightVolumeParamsDictNoTag = typing.TypedDict('_VolumeWeightedStatsWeightVolumeParamsDictNoTag', {
    "weight-volume": InputPathType,
    "match-maps": bool,
})
VolumeWeightedStatsWeightVolumeParamsDictTagged = typing.TypedDict('VolumeWeightedStatsWeightVolumeParamsDictTagged', {
    "@type": typing.Literal["weight-volume"],
    "weight-volume": InputPathType,
    "match-maps": bool,
})
VolumeWeightedStatsWeightVolumeParamsDict = _VolumeWeightedStatsWeightVolumeParamsDictNoTag | VolumeWeightedStatsWeightVolumeParamsDictTagged


_VolumeWeightedStatsRoiParamsDictNoTag = typing.TypedDict('_VolumeWeightedStatsRoiParamsDictNoTag', {
    "roi-volume": InputPathType,
    "match-maps": bool,
})
VolumeWeightedStatsRoiParamsDictTagged = typing.TypedDict('VolumeWeightedStatsRoiParamsDictTagged', {
    "@type": typing.Literal["roi"],
    "roi-volume": InputPathType,
    "match-maps": bool,
})
VolumeWeightedStatsRoiParamsDict = _VolumeWeightedStatsRoiParamsDictNoTag | VolumeWeightedStatsRoiParamsDictTagged


_VolumeWeightedStatsParamsDictNoTag = typing.TypedDict('_VolumeWeightedStatsParamsDictNoTag', {
    "volume-in": InputPathType,
    "weight-volume": typing.NotRequired[VolumeWeightedStatsWeightVolumeParamsDict | None],
    "roi": typing.NotRequired[VolumeWeightedStatsRoiParamsDict | None],
    "percent": typing.NotRequired[float | None],
    "sample": typing.NotRequired[bool | None],
    "subvolume": typing.NotRequired[str | None],
    "show-map-name": bool,
    "sum": bool,
    "mean": bool,
})
VolumeWeightedStatsParamsDictTagged = typing.TypedDict('VolumeWeightedStatsParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-weighted-stats"],
    "volume-in": InputPathType,
    "weight-volume": typing.NotRequired[VolumeWeightedStatsWeightVolumeParamsDict | None],
    "roi": typing.NotRequired[VolumeWeightedStatsRoiParamsDict | None],
    "percent": typing.NotRequired[float | None],
    "sample": typing.NotRequired[bool | None],
    "subvolume": typing.NotRequired[str | None],
    "show-map-name": bool,
    "sum": bool,
    "mean": bool,
})
VolumeWeightedStatsParamsDict = _VolumeWeightedStatsParamsDictNoTag | VolumeWeightedStatsParamsDictTagged


def volume_weighted_stats_weight_volume(
    weight_volume: InputPathType,
    match_maps: bool = False,
) -> VolumeWeightedStatsWeightVolumeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        weight_volume: volume file containing the weights.
        match_maps: each subvolume of input uses the corresponding subvolume\
            from the weights file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "weight-volume",
        "weight-volume": weight_volume,
        "match-maps": match_maps,
    }
    return params


def volume_weighted_stats_weight_volume_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeWeightedStatsWeightVolumeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("weight-volume", None) is None:
        raise StyxValidationError("`weight-volume` must not be None")
    if not isinstance(params["weight-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`weight-volume` has the wrong type: Received `{type(params.get("weight-volume", None))}` expected `InputPathType`')
    if params.get("match-maps", False) is None:
        raise StyxValidationError("`match-maps` must not be None")
    if not isinstance(params["match-maps"], bool):
        raise StyxValidationError(f'`match-maps` has the wrong type: Received `{type(params.get("match-maps", False))}` expected `bool`')


def volume_weighted_stats_weight_volume_cargs(
    params: VolumeWeightedStatsWeightVolumeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-weight-volume",
        execution.input_file(params.get("weight-volume", None))
    ])
    if params.get("match-maps", False):
        cargs.append("-match-maps")
    return cargs


def volume_weighted_stats_roi(
    roi_volume: InputPathType,
    match_maps: bool = False,
) -> VolumeWeightedStatsRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_volume: the roi, as a volume file.
        match_maps: each subvolume of input uses the corresponding subvolume\
            from the roi file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-volume": roi_volume,
        "match-maps": match_maps,
    }
    return params


def volume_weighted_stats_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeWeightedStatsRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-volume", None) is None:
        raise StyxValidationError("`roi-volume` must not be None")
    if not isinstance(params["roi-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType`')
    if params.get("match-maps", False) is None:
        raise StyxValidationError("`match-maps` must not be None")
    if not isinstance(params["match-maps"], bool):
        raise StyxValidationError(f'`match-maps` has the wrong type: Received `{type(params.get("match-maps", False))}` expected `bool`')


def volume_weighted_stats_roi_cargs(
    params: VolumeWeightedStatsRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        execution.input_file(params.get("roi-volume", None))
    ])
    if params.get("match-maps", False):
        cargs.append("-match-maps")
    return cargs


class VolumeWeightedStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeWeightedStatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_weighted_stats_params(
    volume_in: InputPathType,
    weight_volume: VolumeWeightedStatsWeightVolumeParamsDict | None = None,
    roi: VolumeWeightedStatsRoiParamsDict | None = None,
    percent: float | None = None,
    sample: bool | None = None,
    subvolume: str | None = None,
    show_map_name: bool = False,
    sum_: bool = False,
    mean: bool = False,
) -> VolumeWeightedStatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        weight_volume: use weights from a volume file.
        roi: only consider data inside an roi.
        percent: compute weighted percentile\
            \
            the percentile to find, must be between 0 and 100.
        sample: compute weighted standard deviation\
            \
            estimate population stdev from the sample.
        subvolume: only display output for one subvolume\
            \
            the subvolume number or name.
        show_map_name: print map index and name before each output.
        sum_: compute weighted sum.
        mean: compute weighted mean.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-weighted-stats",
        "volume-in": volume_in,
        "show-map-name": show_map_name,
        "sum": sum_,
        "mean": mean,
    }
    if weight_volume is not None:
        params["weight-volume"] = weight_volume
    if roi is not None:
        params["roi"] = roi
    if percent is not None:
        params["percent"] = percent
    if sample is not None:
        params["sample"] = sample
    if subvolume is not None:
        params["subvolume"] = subvolume
    return params


def volume_weighted_stats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeWeightedStatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("weight-volume", None) is not None:
        volume_weighted_stats_weight_volume_validate(params["weight-volume"])
    if params.get("roi", None) is not None:
        volume_weighted_stats_roi_validate(params["roi"])
    if params.get("percent", None) is not None:
        if not isinstance(params["percent"], (float, int)):
            raise StyxValidationError(f'`percent` has the wrong type: Received `{type(params.get("percent", None))}` expected `float | None`')
    if params.get("sample", None) is not None:
        if not isinstance(params["sample"], bool):
            raise StyxValidationError(f'`sample` has the wrong type: Received `{type(params.get("sample", None))}` expected `bool | None`')
    if params.get("subvolume", None) is not None:
        if not isinstance(params["subvolume"], str):
            raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `str | None`')
    if params.get("show-map-name", False) is None:
        raise StyxValidationError("`show-map-name` must not be None")
    if not isinstance(params["show-map-name"], bool):
        raise StyxValidationError(f'`show-map-name` has the wrong type: Received `{type(params.get("show-map-name", False))}` expected `bool`')
    if params.get("sum", False) is None:
        raise StyxValidationError("`sum` must not be None")
    if not isinstance(params["sum"], bool):
        raise StyxValidationError(f'`sum` has the wrong type: Received `{type(params.get("sum", False))}` expected `bool`')
    if params.get("mean", False) is None:
        raise StyxValidationError("`mean` must not be None")
    if not isinstance(params["mean"], bool):
        raise StyxValidationError(f'`mean` has the wrong type: Received `{type(params.get("mean", False))}` expected `bool`')


def volume_weighted_stats_cargs(
    params: VolumeWeightedStatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-weighted-stats"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    if params.get("weight-volume", None) is not None or params.get("roi", None) is not None:
        cargs.extend([
            *(volume_weighted_stats_weight_volume_cargs(params.get("weight-volume", None), execution) if (params.get("weight-volume", None) is not None) else []),
            *(volume_weighted_stats_roi_cargs(params.get("roi", None), execution) if (params.get("roi", None) is not None) else [])
        ])
    if params.get("percent", None) is not None:
        cargs.extend([
            "-percentile",
            str(params.get("percent", None))
        ])
    if params.get("sample", None) is not None:
        cargs.extend([
            "-stdev",
            "-sample"
        ])
    if params.get("subvolume", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvolume", None)
        ])
    if params.get("show-map-name", False):
        cargs.append("-show-map-name")
    if params.get("sum", False):
        cargs.append("-sum")
    if params.get("mean", False):
        cargs.append("-mean")
    return cargs


def volume_weighted_stats_outputs(
    params: VolumeWeightedStatsParamsDict,
    execution: Execution,
) -> VolumeWeightedStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeWeightedStatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_weighted_stats_execute(
    params: VolumeWeightedStatsParamsDict,
    runner: Runner | None = None,
) -> VolumeWeightedStatsOutputs:
    """
    WEIGHTED SPATIAL STATISTICS ON A VOLUME FILE.
    
    For each subvolume of the input, a line of text is printed, resulting from
    the specified operation. If -weight-volume is not specified, each voxel's
    volume is used. Use -subvolume to only give output for a single subvolume.
    If the -roi option is used without -match-maps, then each line will contain
    as many numbers as there are maps in the ROI file, separated by tab
    characters. Exactly one of -mean, -stdev, -percentile or -sum must be
    specified.
    
    Using -sum without -weight-volume is equivalent to integrating with respect
    to volume.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWeightedStatsOutputs`).
    """
    volume_weighted_stats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_WEIGHTED_STATS_METADATA)
    params = execution.params(params)
    cargs = volume_weighted_stats_cargs(params, execution)
    ret = volume_weighted_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_weighted_stats(
    volume_in: InputPathType,
    weight_volume: VolumeWeightedStatsWeightVolumeParamsDict | None = None,
    roi: VolumeWeightedStatsRoiParamsDict | None = None,
    percent: float | None = None,
    sample: bool | None = None,
    subvolume: str | None = None,
    show_map_name: bool = False,
    sum_: bool = False,
    mean: bool = False,
    runner: Runner | None = None,
) -> VolumeWeightedStatsOutputs:
    """
    WEIGHTED SPATIAL STATISTICS ON A VOLUME FILE.
    
    For each subvolume of the input, a line of text is printed, resulting from
    the specified operation. If -weight-volume is not specified, each voxel's
    volume is used. Use -subvolume to only give output for a single subvolume.
    If the -roi option is used without -match-maps, then each line will contain
    as many numbers as there are maps in the ROI file, separated by tab
    characters. Exactly one of -mean, -stdev, -percentile or -sum must be
    specified.
    
    Using -sum without -weight-volume is equivalent to integrating with respect
    to volume.
    
    Args:
        volume_in: the input volume.
        weight_volume: use weights from a volume file.
        roi: only consider data inside an roi.
        percent: compute weighted percentile\
            \
            the percentile to find, must be between 0 and 100.
        sample: compute weighted standard deviation\
            \
            estimate population stdev from the sample.
        subvolume: only display output for one subvolume\
            \
            the subvolume number or name.
        show_map_name: print map index and name before each output.
        sum_: compute weighted sum.
        mean: compute weighted mean.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWeightedStatsOutputs`).
    """
    params = volume_weighted_stats_params(
        volume_in=volume_in,
        weight_volume=weight_volume,
        roi=roi,
        percent=percent,
        sample=sample,
        subvolume=subvolume,
        show_map_name=show_map_name,
        sum_=sum_,
        mean=mean,
    )
    return volume_weighted_stats_execute(params, runner)


__all__ = [
    "VOLUME_WEIGHTED_STATS_METADATA",
    "VolumeWeightedStatsOutputs",
    "VolumeWeightedStatsParamsDict",
    "VolumeWeightedStatsParamsDictTagged",
    "VolumeWeightedStatsRoiParamsDict",
    "VolumeWeightedStatsRoiParamsDictTagged",
    "VolumeWeightedStatsWeightVolumeParamsDict",
    "VolumeWeightedStatsWeightVolumeParamsDictTagged",
    "volume_weighted_stats",
    "volume_weighted_stats_execute",
    "volume_weighted_stats_params",
    "volume_weighted_stats_roi",
    "volume_weighted_stats_weight_volume",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_GEODESIC_ROIS_METADATA = Metadata(
    id="ec6694baabf8b66b3eafbe5048653b2c790dbe3c.workbench",
    name="surface-geodesic-rois",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceGeodesicRoisParamsDictNoTag = typing.TypedDict('_SurfaceGeodesicRoisParamsDictNoTag', {
    "surface": InputPathType,
    "limit": float,
    "vertex-list-file": str,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "name-list-file": typing.NotRequired[str | None],
    "method": typing.NotRequired[str | None],
    "sigma": typing.NotRequired[float | None],
})
SurfaceGeodesicRoisParamsDictTagged = typing.TypedDict('SurfaceGeodesicRoisParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-geodesic-rois"],
    "surface": InputPathType,
    "limit": float,
    "vertex-list-file": str,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "name-list-file": typing.NotRequired[str | None],
    "method": typing.NotRequired[str | None],
    "sigma": typing.NotRequired[float | None],
})
SurfaceGeodesicRoisParamsDict = _SurfaceGeodesicRoisParamsDictNoTag | SurfaceGeodesicRoisParamsDictTagged


class SurfaceGeodesicRoisOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceGeodesicRoisParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def surface_geodesic_rois_params(
    surface: InputPathType,
    limit: float,
    vertex_list_file: str,
    metric_out: str,
    area_metric: InputPathType | None = None,
    name_list_file: str | None = None,
    method: str | None = None,
    sigma: float | None = None,
) -> SurfaceGeodesicRoisParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to draw on.
        limit: geodesic distance limit from vertex, in mm.
        vertex_list_file: a text file containing the vertices to draw ROIs\
            around.
        metric_out: the output metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        name_list_file: name the columns from text file\
            \
            a text file containing column names, one per line.
        method: how to handle overlapping ROIs, default ALLOW\
            \
            the method of resolving overlaps.
        sigma: generate a gaussian kernel instead of a flat ROI\
            \
            the sigma for the gaussian kernel, in mm.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-geodesic-rois",
        "surface": surface,
        "limit": limit,
        "vertex-list-file": vertex_list_file,
        "metric-out": metric_out,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    if name_list_file is not None:
        params["name-list-file"] = name_list_file
    if method is not None:
        params["method"] = method
    if sigma is not None:
        params["sigma"] = sigma
    return params


def surface_geodesic_rois_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceGeodesicRoisParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("limit", None) is None:
        raise StyxValidationError("`limit` must not be None")
    if not isinstance(params["limit"], (float, int)):
        raise StyxValidationError(f'`limit` has the wrong type: Received `{type(params.get("limit", None))}` expected `float`')
    if params.get("vertex-list-file", None) is None:
        raise StyxValidationError("`vertex-list-file` must not be None")
    if not isinstance(params["vertex-list-file"], str):
        raise StyxValidationError(f'`vertex-list-file` has the wrong type: Received `{type(params.get("vertex-list-file", None))}` expected `str`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("name-list-file", None) is not None:
        if not isinstance(params["name-list-file"], str):
            raise StyxValidationError(f'`name-list-file` has the wrong type: Received `{type(params.get("name-list-file", None))}` expected `str | None`')
    if params.get("method", None) is not None:
        if not isinstance(params["method"], str):
            raise StyxValidationError(f'`method` has the wrong type: Received `{type(params.get("method", None))}` expected `str | None`')
    if params.get("sigma", None) is not None:
        if not isinstance(params["sigma"], (float, int)):
            raise StyxValidationError(f'`sigma` has the wrong type: Received `{type(params.get("sigma", None))}` expected `float | None`')


def surface_geodesic_rois_cargs(
    params: SurfaceGeodesicRoisParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-geodesic-rois"
    ])
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("limit", None)))
    cargs.append(params.get("vertex-list-file", None))
    cargs.append(params.get("metric-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("name-list-file", None) is not None:
        cargs.extend([
            "-names",
            params.get("name-list-file", None)
        ])
    if params.get("method", None) is not None:
        cargs.extend([
            "-overlap-logic",
            params.get("method", None)
        ])
    if params.get("sigma", None) is not None:
        cargs.extend([
            "-gaussian",
            str(params.get("sigma", None))
        ])
    return cargs


def surface_geodesic_rois_outputs(
    params: SurfaceGeodesicRoisParamsDict,
    execution: Execution,
) -> SurfaceGeodesicRoisOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceGeodesicRoisOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def surface_geodesic_rois_execute(
    params: SurfaceGeodesicRoisParamsDict,
    runner: Runner | None = None,
) -> SurfaceGeodesicRoisOutputs:
    """
    DRAW GEODESIC LIMITED ROIS AT VERTICES.
    
    For each vertex in the list file, a column in the output metric is created,
    and an ROI around that vertex is drawn in that column. Each metric column
    will have zeros outside the geodesic distance spacified by <limit>, and by
    default will have a value of 1.0 inside it. If the -gaussian option is
    specified, the values inside the ROI will instead form a gaussian with the
    specified value of sigma, normalized so that the sum of the nonzero values
    in the metric column is 1.0. The <method> argument to -overlap-logic must be
    one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means that ROIs
    are treated independently and may overlap. CLOSEST means that ROIs may not
    overlap, and that no ROI contains vertices that are closer to a different
    seed vertex. EXCLUDE means that ROIs may not overlap, and that any vertex
    within range of more than one ROI does not belong to any ROI.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicRoisOutputs`).
    """
    surface_geodesic_rois_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_GEODESIC_ROIS_METADATA)
    params = execution.params(params)
    cargs = surface_geodesic_rois_cargs(params, execution)
    ret = surface_geodesic_rois_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_geodesic_rois(
    surface: InputPathType,
    limit: float,
    vertex_list_file: str,
    metric_out: str,
    area_metric: InputPathType | None = None,
    name_list_file: str | None = None,
    method: str | None = None,
    sigma: float | None = None,
    runner: Runner | None = None,
) -> SurfaceGeodesicRoisOutputs:
    """
    DRAW GEODESIC LIMITED ROIS AT VERTICES.
    
    For each vertex in the list file, a column in the output metric is created,
    and an ROI around that vertex is drawn in that column. Each metric column
    will have zeros outside the geodesic distance spacified by <limit>, and by
    default will have a value of 1.0 inside it. If the -gaussian option is
    specified, the values inside the ROI will instead form a gaussian with the
    specified value of sigma, normalized so that the sum of the nonzero values
    in the metric column is 1.0. The <method> argument to -overlap-logic must be
    one of ALLOW, CLOSEST, or EXCLUDE. ALLOW is the default, and means that ROIs
    are treated independently and may overlap. CLOSEST means that ROIs may not
    overlap, and that no ROI contains vertices that are closer to a different
    seed vertex. EXCLUDE means that ROIs may not overlap, and that any vertex
    within range of more than one ROI does not belong to any ROI.
    
    Args:
        surface: the surface to draw on.
        limit: geodesic distance limit from vertex, in mm.
        vertex_list_file: a text file containing the vertices to draw ROIs\
            around.
        metric_out: the output metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        name_list_file: name the columns from text file\
            \
            a text file containing column names, one per line.
        method: how to handle overlapping ROIs, default ALLOW\
            \
            the method of resolving overlaps.
        sigma: generate a gaussian kernel instead of a flat ROI\
            \
            the sigma for the gaussian kernel, in mm.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceGeodesicRoisOutputs`).
    """
    params = surface_geodesic_rois_params(
        surface=surface,
        limit=limit,
        vertex_list_file=vertex_list_file,
        metric_out=metric_out,
        area_metric=area_metric,
        name_list_file=name_list_file,
        method=method,
        sigma=sigma,
    )
    return surface_geodesic_rois_execute(params, runner)


__all__ = [
    "SURFACE_GEODESIC_ROIS_METADATA",
    "SurfaceGeodesicRoisOutputs",
    "SurfaceGeodesicRoisParamsDict",
    "SurfaceGeodesicRoisParamsDictTagged",
    "surface_geodesic_rois",
    "surface_geodesic_rois_execute",
    "surface_geodesic_rois_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SCENE_CAPTURE_IMAGE_METADATA = Metadata(
    id="d7f78e620a92e4692790ceda0d24ff04fb837ed8.workbench",
    name="scene-capture-image",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SceneCaptureImageSizeWidthHeightParamsDictNoTag = typing.TypedDict('_SceneCaptureImageSizeWidthHeightParamsDictNoTag', {
    "width": float,
    "height": float,
})
SceneCaptureImageSizeWidthHeightParamsDictTagged = typing.TypedDict('SceneCaptureImageSizeWidthHeightParamsDictTagged', {
    "@type": typing.Literal["size-width-height"],
    "width": float,
    "height": float,
})
SceneCaptureImageSizeWidthHeightParamsDict = _SceneCaptureImageSizeWidthHeightParamsDictNoTag | SceneCaptureImageSizeWidthHeightParamsDictTagged


_SceneCaptureImageResolutionParamsDictNoTag = typing.TypedDict('_SceneCaptureImageResolutionParamsDictNoTag', {
    "Number of pixels": float,
    "Units Name": str,
})
SceneCaptureImageResolutionParamsDictTagged = typing.TypedDict('SceneCaptureImageResolutionParamsDictTagged', {
    "@type": typing.Literal["resolution"],
    "Number of pixels": float,
    "Units Name": str,
})
SceneCaptureImageResolutionParamsDict = _SceneCaptureImageResolutionParamsDictNoTag | SceneCaptureImageResolutionParamsDictTagged


_SceneCaptureImageSetMapYokeParamsDictNoTag = typing.TypedDict('_SceneCaptureImageSetMapYokeParamsDictNoTag', {
    "map yoking roman numeral": str,
    "map undex": int,
})
SceneCaptureImageSetMapYokeParamsDictTagged = typing.TypedDict('SceneCaptureImageSetMapYokeParamsDictTagged', {
    "@type": typing.Literal["set-map-yoke"],
    "map yoking roman numeral": str,
    "map undex": int,
})
SceneCaptureImageSetMapYokeParamsDict = _SceneCaptureImageSetMapYokeParamsDictNoTag | SceneCaptureImageSetMapYokeParamsDictTagged


_SceneCaptureImageConnDbLoginParamsDictNoTag = typing.TypedDict('_SceneCaptureImageConnDbLoginParamsDictNoTag', {
    "username": str,
    "password": str,
})
SceneCaptureImageConnDbLoginParamsDictTagged = typing.TypedDict('SceneCaptureImageConnDbLoginParamsDictTagged', {
    "@type": typing.Literal["conn-db-login"],
    "username": str,
    "password": str,
})
SceneCaptureImageConnDbLoginParamsDict = _SceneCaptureImageConnDbLoginParamsDictNoTag | SceneCaptureImageConnDbLoginParamsDictTagged


_SceneCaptureImageParamsDictNoTag = typing.TypedDict('_SceneCaptureImageParamsDictNoTag', {
    "scene-file": str,
    "scene-name-or-number": str,
    "image-file-name": str,
    "size-width-height": typing.NotRequired[SceneCaptureImageSizeWidthHeightParamsDict | None],
    "resolution": typing.NotRequired[SceneCaptureImageResolutionParamsDict | None],
    "set-map-yoke": typing.NotRequired[SceneCaptureImageSetMapYokeParamsDict | None],
    "conn-db-login": typing.NotRequired[SceneCaptureImageConnDbLoginParamsDict | None],
    "Renderer": typing.NotRequired[str | None],
    "size": typing.NotRequired[int | None],
    "units": typing.NotRequired[str | None],
    "height": typing.NotRequired[float | None],
    "width": typing.NotRequired[float | None],
    "print-image-info": bool,
    "show-capture-settings": bool,
    "no-scene-colors": bool,
    "size-capture": bool,
    "size-window": bool,
})
SceneCaptureImageParamsDictTagged = typing.TypedDict('SceneCaptureImageParamsDictTagged', {
    "@type": typing.Literal["workbench/scene-capture-image"],
    "scene-file": str,
    "scene-name-or-number": str,
    "image-file-name": str,
    "size-width-height": typing.NotRequired[SceneCaptureImageSizeWidthHeightParamsDict | None],
    "resolution": typing.NotRequired[SceneCaptureImageResolutionParamsDict | None],
    "set-map-yoke": typing.NotRequired[SceneCaptureImageSetMapYokeParamsDict | None],
    "conn-db-login": typing.NotRequired[SceneCaptureImageConnDbLoginParamsDict | None],
    "Renderer": typing.NotRequired[str | None],
    "size": typing.NotRequired[int | None],
    "units": typing.NotRequired[str | None],
    "height": typing.NotRequired[float | None],
    "width": typing.NotRequired[float | None],
    "print-image-info": bool,
    "show-capture-settings": bool,
    "no-scene-colors": bool,
    "size-capture": bool,
    "size-window": bool,
})
SceneCaptureImageParamsDict = _SceneCaptureImageParamsDictNoTag | SceneCaptureImageParamsDictTagged


def scene_capture_image_size_width_height(
    width: float,
    height: float,
) -> SceneCaptureImageSizeWidthHeightParamsDictTagged:
    """
    Build parameters.
    
    Args:
        width: Width for output image.
        height: Height for output image.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "size-width-height",
        "width": width,
        "height": height,
    }
    return params


def scene_capture_image_size_width_height_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneCaptureImageSizeWidthHeightParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("width", None) is None:
        raise StyxValidationError("`width` must not be None")
    if not isinstance(params["width"], (float, int)):
        raise StyxValidationError(f'`width` has the wrong type: Received `{type(params.get("width", None))}` expected `float`')
    if params.get("height", None) is None:
        raise StyxValidationError("`height` must not be None")
    if not isinstance(params["height"], (float, int)):
        raise StyxValidationError(f'`height` has the wrong type: Received `{type(params.get("height", None))}` expected `float`')


def scene_capture_image_size_width_height_cargs(
    params: SceneCaptureImageSizeWidthHeightParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-size-width-height",
        str(params.get("width", None)),
        str(params.get("height", None))
    ])
    return cargs


def scene_capture_image_resolution(
    number_of_pixels: float,
    units_name: str,
) -> SceneCaptureImageResolutionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        number_of_pixels: number of pixels.
        units_name: Name of resolution units. Valid resolution unit names are:\
            PIXELS_PER_INCH\
            PIXELS_PER_CENTIMETER\
            PIXELS_PER_METER\
            PIXELS_PER_MILLIMETER.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "resolution",
        "Number of pixels": number_of_pixels,
        "Units Name": units_name,
    }
    return params


def scene_capture_image_resolution_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneCaptureImageResolutionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("Number of pixels", None) is None:
        raise StyxValidationError("`Number of pixels` must not be None")
    if not isinstance(params["Number of pixels"], (float, int)):
        raise StyxValidationError(f'`Number of pixels` has the wrong type: Received `{type(params.get("Number of pixels", None))}` expected `float`')
    if params.get("Units Name", None) is None:
        raise StyxValidationError("`Units Name` must not be None")
    if not isinstance(params["Units Name"], str):
        raise StyxValidationError(f'`Units Name` has the wrong type: Received `{type(params.get("Units Name", None))}` expected `str`')


def scene_capture_image_resolution_cargs(
    params: SceneCaptureImageResolutionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-resolution",
        str(params.get("Number of pixels", None)),
        params.get("Units Name", None)
    ])
    return cargs


def scene_capture_image_set_map_yoke(
    map_yoking_roman_numeral: str,
    map_undex: int,
) -> SceneCaptureImageSetMapYokeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        map_yoking_roman_numeral: Roman numeral identifying the map yoking\
            group (I, II, III, IV, V, VI, VII, VIII, IX, X).
        map_undex: Map index for yoking group. Indices start at 1 (one).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "set-map-yoke",
        "map yoking roman numeral": map_yoking_roman_numeral,
        "map undex": map_undex,
    }
    return params


def scene_capture_image_set_map_yoke_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneCaptureImageSetMapYokeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("map yoking roman numeral", None) is None:
        raise StyxValidationError("`map yoking roman numeral` must not be None")
    if not isinstance(params["map yoking roman numeral"], str):
        raise StyxValidationError(f'`map yoking roman numeral` has the wrong type: Received `{type(params.get("map yoking roman numeral", None))}` expected `str`')
    if params.get("map undex", None) is None:
        raise StyxValidationError("`map undex` must not be None")
    if not isinstance(params["map undex"], int):
        raise StyxValidationError(f'`map undex` has the wrong type: Received `{type(params.get("map undex", None))}` expected `int`')


def scene_capture_image_set_map_yoke_cargs(
    params: SceneCaptureImageSetMapYokeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-set-map-yoke",
        params.get("map yoking roman numeral", None),
        str(params.get("map undex", None))
    ])
    return cargs


def scene_capture_image_conn_db_login(
    username: str,
    password: str,
) -> SceneCaptureImageConnDbLoginParamsDictTagged:
    """
    Build parameters.
    
    Args:
        username: Connectome DB Username.
        password: Connectome DB Password.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "conn-db-login",
        "username": username,
        "password": password,
    }
    return params


def scene_capture_image_conn_db_login_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneCaptureImageConnDbLoginParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("username", None) is None:
        raise StyxValidationError("`username` must not be None")
    if not isinstance(params["username"], str):
        raise StyxValidationError(f'`username` has the wrong type: Received `{type(params.get("username", None))}` expected `str`')
    if params.get("password", None) is None:
        raise StyxValidationError("`password` must not be None")
    if not isinstance(params["password"], str):
        raise StyxValidationError(f'`password` has the wrong type: Received `{type(params.get("password", None))}` expected `str`')


def scene_capture_image_conn_db_login_cargs(
    params: SceneCaptureImageConnDbLoginParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-conn-db-login",
        params.get("username", None),
        params.get("password", None)
    ])
    return cargs


class SceneCaptureImageOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SceneCaptureImageParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def scene_capture_image_params(
    scene_file: str,
    scene_name_or_number: str,
    image_file_name: str,
    size_width_height: SceneCaptureImageSizeWidthHeightParamsDict | None = None,
    resolution: SceneCaptureImageResolutionParamsDict | None = None,
    set_map_yoke: SceneCaptureImageSetMapYokeParamsDict | None = None,
    conn_db_login: SceneCaptureImageConnDbLoginParamsDict | None = None,
    renderer: str | None = None,
    size: int | None = None,
    units: str | None = None,
    height: float | None = None,
    width: float | None = None,
    print_image_info: bool = False,
    show_capture_settings: bool = False,
    no_scene_colors: bool = False,
    size_capture: bool = False,
    size_window: bool = False,
) -> SceneCaptureImageParamsDictTagged:
    """
    Build parameters.
    
    Args:
        scene_file: scene file.
        scene_name_or_number: name or number (starting at one) of the scene in\
            the scene file.
        image_file_name: output - image file name\
            The file name must end with a valid extension that identifies the\
            image file format. Valid extensions on this system are: (.bmp .jpeg\
            .jpg .png .ppm).\
            \
            If there is more than one window in the scene, multiple image files\
            are output with the window's number inserted into the name of the\
            image file immediately before the image file's extension.
        size_width_height: Width and height for output image.
        resolution: Image resolution (number pixels per size unit)\
            Default is 300 PIXELS_PER_INCH.
        set_map_yoke: Override selected map index for a map yoking group.
        conn_db_login: Login for scenes with files in Connectome Database. If\
            this option is not specified, the login and password stored in the\
            user's preferences is used.
        renderer: Select renderer for drawing image\
            \
            Name of renderer to use for drawing image\
            Available renderers are (first is default):\
            OSMesa - Mesa3D OSMesa software renderer.
        size: Add a margin to sides of the image using the window's background\
            color.\
            \
            size of margin, in pixels, added to all sides of output image.
        units: Units for image width/height\
            Default is PIXELS\
            \
            Name of units for image width/height. Valid units are:\
            INCHES\
            CENTIMETERS\
            MILLIMETERS\
            METERS\
            PIXELS.
        height: Height for output image. Width is computed using the aspect\
            ratio from the window's width and height saved in the scene.\
            \
            Height for output image.
        width: Width for output image. Height is computed using the aspect\
            ratio from the window's width and height saved in the scene.\
            \
            Width for output image.
        print_image_info: Print the size and other information about output\
            images only and DO NOT create any output images.
        show_capture_settings: Print settings from Capture Dialog only, DO NOT\
            create image file(s).
        no_scene_colors: Do not use background and foreground colors in scene.
        size_capture: Output image uses size from Capture Dialog when scene was\
            created.
        size_window: Output image is size of window's graphics region from when\
            scene was created.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/scene-capture-image",
        "scene-file": scene_file,
        "scene-name-or-number": scene_name_or_number,
        "image-file-name": image_file_name,
        "print-image-info": print_image_info,
        "show-capture-settings": show_capture_settings,
        "no-scene-colors": no_scene_colors,
        "size-capture": size_capture,
        "size-window": size_window,
    }
    if size_width_height is not None:
        params["size-width-height"] = size_width_height
    if resolution is not None:
        params["resolution"] = resolution
    if set_map_yoke is not None:
        params["set-map-yoke"] = set_map_yoke
    if conn_db_login is not None:
        params["conn-db-login"] = conn_db_login
    if renderer is not None:
        params["Renderer"] = renderer
    if size is not None:
        params["size"] = size
    if units is not None:
        params["units"] = units
    if height is not None:
        params["height"] = height
    if width is not None:
        params["width"] = width
    return params


def scene_capture_image_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SceneCaptureImageParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("scene-file", None) is None:
        raise StyxValidationError("`scene-file` must not be None")
    if not isinstance(params["scene-file"], str):
        raise StyxValidationError(f'`scene-file` has the wrong type: Received `{type(params.get("scene-file", None))}` expected `str`')
    if params.get("scene-name-or-number", None) is None:
        raise StyxValidationError("`scene-name-or-number` must not be None")
    if not isinstance(params["scene-name-or-number"], str):
        raise StyxValidationError(f'`scene-name-or-number` has the wrong type: Received `{type(params.get("scene-name-or-number", None))}` expected `str`')
    if params.get("image-file-name", None) is None:
        raise StyxValidationError("`image-file-name` must not be None")
    if not isinstance(params["image-file-name"], str):
        raise StyxValidationError(f'`image-file-name` has the wrong type: Received `{type(params.get("image-file-name", None))}` expected `str`')
    if params.get("size-width-height", None) is not None:
        scene_capture_image_size_width_height_validate(params["size-width-height"])
    if params.get("resolution", None) is not None:
        scene_capture_image_resolution_validate(params["resolution"])
    if params.get("set-map-yoke", None) is not None:
        scene_capture_image_set_map_yoke_validate(params["set-map-yoke"])
    if params.get("conn-db-login", None) is not None:
        scene_capture_image_conn_db_login_validate(params["conn-db-login"])
    if params.get("Renderer", None) is not None:
        if not isinstance(params["Renderer"], str):
            raise StyxValidationError(f'`Renderer` has the wrong type: Received `{type(params.get("Renderer", None))}` expected `str | None`')
    if params.get("size", None) is not None:
        if not isinstance(params["size"], int):
            raise StyxValidationError(f'`size` has the wrong type: Received `{type(params.get("size", None))}` expected `int | None`')
    if params.get("units", None) is not None:
        if not isinstance(params["units"], str):
            raise StyxValidationError(f'`units` has the wrong type: Received `{type(params.get("units", None))}` expected `str | None`')
    if params.get("height", None) is not None:
        if not isinstance(params["height"], (float, int)):
            raise StyxValidationError(f'`height` has the wrong type: Received `{type(params.get("height", None))}` expected `float | None`')
    if params.get("width", None) is not None:
        if not isinstance(params["width"], (float, int)):
            raise StyxValidationError(f'`width` has the wrong type: Received `{type(params.get("width", None))}` expected `float | None`')
    if params.get("print-image-info", False) is None:
        raise StyxValidationError("`print-image-info` must not be None")
    if not isinstance(params["print-image-info"], bool):
        raise StyxValidationError(f'`print-image-info` has the wrong type: Received `{type(params.get("print-image-info", False))}` expected `bool`')
    if params.get("show-capture-settings", False) is None:
        raise StyxValidationError("`show-capture-settings` must not be None")
    if not isinstance(params["show-capture-settings"], bool):
        raise StyxValidationError(f'`show-capture-settings` has the wrong type: Received `{type(params.get("show-capture-settings", False))}` expected `bool`')
    if params.get("no-scene-colors", False) is None:
        raise StyxValidationError("`no-scene-colors` must not be None")
    if not isinstance(params["no-scene-colors"], bool):
        raise StyxValidationError(f'`no-scene-colors` has the wrong type: Received `{type(params.get("no-scene-colors", False))}` expected `bool`')
    if params.get("size-capture", False) is None:
        raise StyxValidationError("`size-capture` must not be None")
    if not isinstance(params["size-capture"], bool):
        raise StyxValidationError(f'`size-capture` has the wrong type: Received `{type(params.get("size-capture", False))}` expected `bool`')
    if params.get("size-window", False) is None:
        raise StyxValidationError("`size-window` must not be None")
    if not isinstance(params["size-window"], bool):
        raise StyxValidationError(f'`size-window` has the wrong type: Received `{type(params.get("size-window", False))}` expected `bool`')


def scene_capture_image_cargs(
    params: SceneCaptureImageParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-scene-capture-image"
    ])
    cargs.append(params.get("scene-file", None))
    cargs.append(params.get("scene-name-or-number", None))
    cargs.append(params.get("image-file-name", None))
    if params.get("size-width-height", None) is not None or params.get("resolution", None) is not None or params.get("set-map-yoke", None) is not None or params.get("conn-db-login", None) is not None:
        cargs.extend([
            *(scene_capture_image_size_width_height_cargs(params.get("size-width-height", None), execution) if (params.get("size-width-height", None) is not None) else []),
            *(scene_capture_image_resolution_cargs(params.get("resolution", None), execution) if (params.get("resolution", None) is not None) else []),
            *(scene_capture_image_set_map_yoke_cargs(params.get("set-map-yoke", None), execution) if (params.get("set-map-yoke", None) is not None) else []),
            *(scene_capture_image_conn_db_login_cargs(params.get("conn-db-login", None), execution) if (params.get("conn-db-login", None) is not None) else [])
        ])
    if params.get("Renderer", None) is not None:
        cargs.extend([
            "-renderer",
            params.get("Renderer", None)
        ])
    if params.get("size", None) is not None:
        cargs.extend([
            "-margin",
            str(params.get("size", None))
        ])
    if params.get("units", None) is not None:
        cargs.extend([
            "-units",
            params.get("units", None)
        ])
    if params.get("height", None) is not None:
        cargs.extend([
            "-size-height",
            str(params.get("height", None))
        ])
    if params.get("width", None) is not None:
        cargs.extend([
            "-size-width",
            str(params.get("width", None))
        ])
    if params.get("print-image-info", False):
        cargs.append("-print-image-info")
    if params.get("show-capture-settings", False):
        cargs.append("-show-capture-settings")
    if params.get("no-scene-colors", False):
        cargs.append("-no-scene-colors")
    if params.get("size-capture", False):
        cargs.append("-size-capture")
    if params.get("size-window", False):
        cargs.append("-size-window")
    return cargs


def scene_capture_image_outputs(
    params: SceneCaptureImageParamsDict,
    execution: Execution,
) -> SceneCaptureImageOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SceneCaptureImageOutputs(
        root=execution.output_file("."),
    )
    return ret


def scene_capture_image_execute(
    params: SceneCaptureImageParamsDict,
    runner: Runner | None = None,
) -> SceneCaptureImageOutputs:
    """
    OFFSCREEN RENDERING OF SCENE TO AN IMAGE FILE.
    
    ----------------------------------------------------------------------
    
    Render content of browser windows displayed in a scene into image file(s).
    
    If none of the "-size" options are specified, the default is "-size-window"
    (Output image is size of the window that was saved in the scene).
    
    For the "-size" options that accept a width and/or height, the values
    default to number of pixels. To express the width and/or height in physical
    units (inches, centimeters, etc.), use the "-units" option. When physical
    units are used, the pixel width and height are derived using the physical
    width/height and the image resolution (see the "-resolution" option).
    
    Note that scenes created prior to version 1.2 (May 2016) do not contain
    information about the size of the window. Therefore, one must use the
    "-size-width-height" option.
    
    Examples:
    
    Generate an image of the second scene. Width and height of image is width
    and height of window saved in the scene.
    wb_command -scene-capture-image myscene.scene 2 image2.jpg
    
    Generate an image of the second scene with a margin around sides of the
    image. Width and height of image is width and height of window saved in the
    scene.
    wb_command -scene-capture-image myscene.scene 2 image2.jpg -margin 10
    
    Generate an image of the second scene that is 6 inches width with 300 pixels
    per inch. The resulting width is 1800 pixels. The resulting height of the
    image is a function of the width and the aspect ratio (height divided by
    width) of the window size saved in the scene.
    wb_command -scene-capture-image myscene.scene 2 image21.jpg \\
    -size-width 6 -units INCHES -resolution 300 PIXELS_PER_INCH
    
    Print information about the size of the output image for the second scene
    (no image file is created) using a width of 4.5 centimeters.
    wb_command -scene-capture-image myscene.scene 2 test.jpg \\
    -size-width 4.5 -units CENTIMETERS -print-image-info
    
    
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneCaptureImageOutputs`).
    """
    scene_capture_image_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SCENE_CAPTURE_IMAGE_METADATA)
    params = execution.params(params)
    cargs = scene_capture_image_cargs(params, execution)
    ret = scene_capture_image_outputs(params, execution)
    execution.run(cargs)
    return ret


def scene_capture_image(
    scene_file: str,
    scene_name_or_number: str,
    image_file_name: str,
    size_width_height: SceneCaptureImageSizeWidthHeightParamsDict | None = None,
    resolution: SceneCaptureImageResolutionParamsDict | None = None,
    set_map_yoke: SceneCaptureImageSetMapYokeParamsDict | None = None,
    conn_db_login: SceneCaptureImageConnDbLoginParamsDict | None = None,
    renderer: str | None = None,
    size: int | None = None,
    units: str | None = None,
    height: float | None = None,
    width: float | None = None,
    print_image_info: bool = False,
    show_capture_settings: bool = False,
    no_scene_colors: bool = False,
    size_capture: bool = False,
    size_window: bool = False,
    runner: Runner | None = None,
) -> SceneCaptureImageOutputs:
    """
    OFFSCREEN RENDERING OF SCENE TO AN IMAGE FILE.
    
    ----------------------------------------------------------------------
    
    Render content of browser windows displayed in a scene into image file(s).
    
    If none of the "-size" options are specified, the default is "-size-window"
    (Output image is size of the window that was saved in the scene).
    
    For the "-size" options that accept a width and/or height, the values
    default to number of pixels. To express the width and/or height in physical
    units (inches, centimeters, etc.), use the "-units" option. When physical
    units are used, the pixel width and height are derived using the physical
    width/height and the image resolution (see the "-resolution" option).
    
    Note that scenes created prior to version 1.2 (May 2016) do not contain
    information about the size of the window. Therefore, one must use the
    "-size-width-height" option.
    
    Examples:
    
    Generate an image of the second scene. Width and height of image is width
    and height of window saved in the scene.
    wb_command -scene-capture-image myscene.scene 2 image2.jpg
    
    Generate an image of the second scene with a margin around sides of the
    image. Width and height of image is width and height of window saved in the
    scene.
    wb_command -scene-capture-image myscene.scene 2 image2.jpg -margin 10
    
    Generate an image of the second scene that is 6 inches width with 300 pixels
    per inch. The resulting width is 1800 pixels. The resulting height of the
    image is a function of the width and the aspect ratio (height divided by
    width) of the window size saved in the scene.
    wb_command -scene-capture-image myscene.scene 2 image21.jpg \\
    -size-width 6 -units INCHES -resolution 300 PIXELS_PER_INCH
    
    Print information about the size of the output image for the second scene
    (no image file is created) using a width of 4.5 centimeters.
    wb_command -scene-capture-image myscene.scene 2 test.jpg \\
    -size-width 4.5 -units CENTIMETERS -print-image-info
    
    
    .
    
    Args:
        scene_file: scene file.
        scene_name_or_number: name or number (starting at one) of the scene in\
            the scene file.
        image_file_name: output - image file name\
            The file name must end with a valid extension that identifies the\
            image file format. Valid extensions on this system are: (.bmp .jpeg\
            .jpg .png .ppm).\
            \
            If there is more than one window in the scene, multiple image files\
            are output with the window's number inserted into the name of the\
            image file immediately before the image file's extension.
        size_width_height: Width and height for output image.
        resolution: Image resolution (number pixels per size unit)\
            Default is 300 PIXELS_PER_INCH.
        set_map_yoke: Override selected map index for a map yoking group.
        conn_db_login: Login for scenes with files in Connectome Database. If\
            this option is not specified, the login and password stored in the\
            user's preferences is used.
        renderer: Select renderer for drawing image\
            \
            Name of renderer to use for drawing image\
            Available renderers are (first is default):\
            OSMesa - Mesa3D OSMesa software renderer.
        size: Add a margin to sides of the image using the window's background\
            color.\
            \
            size of margin, in pixels, added to all sides of output image.
        units: Units for image width/height\
            Default is PIXELS\
            \
            Name of units for image width/height. Valid units are:\
            INCHES\
            CENTIMETERS\
            MILLIMETERS\
            METERS\
            PIXELS.
        height: Height for output image. Width is computed using the aspect\
            ratio from the window's width and height saved in the scene.\
            \
            Height for output image.
        width: Width for output image. Height is computed using the aspect\
            ratio from the window's width and height saved in the scene.\
            \
            Width for output image.
        print_image_info: Print the size and other information about output\
            images only and DO NOT create any output images.
        show_capture_settings: Print settings from Capture Dialog only, DO NOT\
            create image file(s).
        no_scene_colors: Do not use background and foreground colors in scene.
        size_capture: Output image uses size from Capture Dialog when scene was\
            created.
        size_window: Output image is size of window's graphics region from when\
            scene was created.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SceneCaptureImageOutputs`).
    """
    params = scene_capture_image_params(
        scene_file=scene_file,
        scene_name_or_number=scene_name_or_number,
        image_file_name=image_file_name,
        size_width_height=size_width_height,
        resolution=resolution,
        set_map_yoke=set_map_yoke,
        conn_db_login=conn_db_login,
        renderer=renderer,
        size=size,
        units=units,
        height=height,
        width=width,
        print_image_info=print_image_info,
        show_capture_settings=show_capture_settings,
        no_scene_colors=no_scene_colors,
        size_capture=size_capture,
        size_window=size_window,
    )
    return scene_capture_image_execute(params, runner)


__all__ = [
    "SCENE_CAPTURE_IMAGE_METADATA",
    "SceneCaptureImageConnDbLoginParamsDict",
    "SceneCaptureImageConnDbLoginParamsDictTagged",
    "SceneCaptureImageOutputs",
    "SceneCaptureImageParamsDict",
    "SceneCaptureImageParamsDictTagged",
    "SceneCaptureImageResolutionParamsDict",
    "SceneCaptureImageResolutionParamsDictTagged",
    "SceneCaptureImageSetMapYokeParamsDict",
    "SceneCaptureImageSetMapYokeParamsDictTagged",
    "SceneCaptureImageSizeWidthHeightParamsDict",
    "SceneCaptureImageSizeWidthHeightParamsDictTagged",
    "scene_capture_image",
    "scene_capture_image_conn_db_login",
    "scene_capture_image_execute",
    "scene_capture_image_params",
    "scene_capture_image_resolution",
    "scene_capture_image_set_map_yoke",
    "scene_capture_image_size_width_height",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_FALSE_CORRELATION_METADATA = Metadata(
    id="4934b43c48754a15f2680766bf60fde0c8338b6e.workbench",
    name="cifti-false-correlation",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiFalseCorrelationLeftSurfaceParamsDictNoTag = typing.TypedDict('_CiftiFalseCorrelationLeftSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "text-out": typing.NotRequired[str | None],
})
CiftiFalseCorrelationLeftSurfaceParamsDictTagged = typing.TypedDict('CiftiFalseCorrelationLeftSurfaceParamsDictTagged', {
    "@type": typing.Literal["left-surface"],
    "surface": InputPathType,
    "text-out": typing.NotRequired[str | None],
})
CiftiFalseCorrelationLeftSurfaceParamsDict = _CiftiFalseCorrelationLeftSurfaceParamsDictNoTag | CiftiFalseCorrelationLeftSurfaceParamsDictTagged


_CiftiFalseCorrelationRightSurfaceParamsDictNoTag = typing.TypedDict('_CiftiFalseCorrelationRightSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "text-out": typing.NotRequired[str | None],
})
CiftiFalseCorrelationRightSurfaceParamsDictTagged = typing.TypedDict('CiftiFalseCorrelationRightSurfaceParamsDictTagged', {
    "@type": typing.Literal["right-surface"],
    "surface": InputPathType,
    "text-out": typing.NotRequired[str | None],
})
CiftiFalseCorrelationRightSurfaceParamsDict = _CiftiFalseCorrelationRightSurfaceParamsDictNoTag | CiftiFalseCorrelationRightSurfaceParamsDictTagged


_CiftiFalseCorrelationCerebellumSurfaceParamsDictNoTag = typing.TypedDict('_CiftiFalseCorrelationCerebellumSurfaceParamsDictNoTag', {
    "surface": InputPathType,
    "text-out": typing.NotRequired[str | None],
})
CiftiFalseCorrelationCerebellumSurfaceParamsDictTagged = typing.TypedDict('CiftiFalseCorrelationCerebellumSurfaceParamsDictTagged', {
    "@type": typing.Literal["cerebellum-surface"],
    "surface": InputPathType,
    "text-out": typing.NotRequired[str | None],
})
CiftiFalseCorrelationCerebellumSurfaceParamsDict = _CiftiFalseCorrelationCerebellumSurfaceParamsDictNoTag | CiftiFalseCorrelationCerebellumSurfaceParamsDictTagged


_CiftiFalseCorrelationParamsDictNoTag = typing.TypedDict('_CiftiFalseCorrelationParamsDictNoTag', {
    "cifti-in": InputPathType,
    "3D-dist": float,
    "geo-outer": float,
    "geo-inner": float,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiFalseCorrelationLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiFalseCorrelationRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiFalseCorrelationCerebellumSurfaceParamsDict | None],
})
CiftiFalseCorrelationParamsDictTagged = typing.TypedDict('CiftiFalseCorrelationParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-false-correlation"],
    "cifti-in": InputPathType,
    "3D-dist": float,
    "geo-outer": float,
    "geo-inner": float,
    "cifti-out": str,
    "left-surface": typing.NotRequired[CiftiFalseCorrelationLeftSurfaceParamsDict | None],
    "right-surface": typing.NotRequired[CiftiFalseCorrelationRightSurfaceParamsDict | None],
    "cerebellum-surface": typing.NotRequired[CiftiFalseCorrelationCerebellumSurfaceParamsDict | None],
})
CiftiFalseCorrelationParamsDict = _CiftiFalseCorrelationParamsDictNoTag | CiftiFalseCorrelationParamsDictTagged


def cifti_false_correlation_left_surface(
    surface: InputPathType,
    text_out: str | None = None,
) -> CiftiFalseCorrelationLeftSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the left surface file.
        text_out: dump the raw measures used to a text file\
            \
            the output text file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "left-surface",
        "surface": surface,
    }
    if text_out is not None:
        params["text-out"] = text_out
    return params


def cifti_false_correlation_left_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiFalseCorrelationLeftSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("text-out", None) is not None:
        if not isinstance(params["text-out"], str):
            raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str | None`')


def cifti_false_correlation_left_surface_cargs(
    params: CiftiFalseCorrelationLeftSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-left-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("text-out", None) is not None:
        cargs.extend([
            "-dump-text",
            params.get("text-out", None)
        ])
    return cargs


def cifti_false_correlation_right_surface(
    surface: InputPathType,
    text_out: str | None = None,
) -> CiftiFalseCorrelationRightSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the right surface file.
        text_out: dump the raw measures used to a text file\
            \
            the output text file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "right-surface",
        "surface": surface,
    }
    if text_out is not None:
        params["text-out"] = text_out
    return params


def cifti_false_correlation_right_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiFalseCorrelationRightSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("text-out", None) is not None:
        if not isinstance(params["text-out"], str):
            raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str | None`')


def cifti_false_correlation_right_surface_cargs(
    params: CiftiFalseCorrelationRightSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-right-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("text-out", None) is not None:
        cargs.extend([
            "-dump-text",
            params.get("text-out", None)
        ])
    return cargs


def cifti_false_correlation_cerebellum_surface(
    surface: InputPathType,
    text_out: str | None = None,
) -> CiftiFalseCorrelationCerebellumSurfaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the cerebellum surface file.
        text_out: dump the raw measures used to a text file\
            \
            the output text file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "cerebellum-surface",
        "surface": surface,
    }
    if text_out is not None:
        params["text-out"] = text_out
    return params


def cifti_false_correlation_cerebellum_surface_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiFalseCorrelationCerebellumSurfaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("text-out", None) is not None:
        if not isinstance(params["text-out"], str):
            raise StyxValidationError(f'`text-out` has the wrong type: Received `{type(params.get("text-out", None))}` expected `str | None`')


def cifti_false_correlation_cerebellum_surface_cargs(
    params: CiftiFalseCorrelationCerebellumSurfaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-cerebellum-surface",
        execution.input_file(params.get("surface", None))
    ])
    if params.get("text-out", None) is not None:
        cargs.extend([
            "-dump-text",
            params.get("text-out", None)
        ])
    return cargs


class CiftiFalseCorrelationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiFalseCorrelationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti dscalar file"""


def cifti_false_correlation_params(
    cifti_in: InputPathType,
    v_3_d_dist: float,
    geo_outer: float,
    geo_inner: float,
    cifti_out: str,
    left_surface: CiftiFalseCorrelationLeftSurfaceParamsDict | None = None,
    right_surface: CiftiFalseCorrelationRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiFalseCorrelationCerebellumSurfaceParamsDict | None = None,
) -> CiftiFalseCorrelationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the cifti file to use for correlation.
        v_3_d_dist: maximum 3D distance to check around each vertex.
        geo_outer: maximum geodesic distance to use for neighboring correlation.
        geo_inner: minimum geodesic distance to use for neighboring correlation.
        cifti_out: the output cifti dscalar file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-false-correlation",
        "cifti-in": cifti_in,
        "3D-dist": v_3_d_dist,
        "geo-outer": geo_outer,
        "geo-inner": geo_inner,
        "cifti-out": cifti_out,
    }
    if left_surface is not None:
        params["left-surface"] = left_surface
    if right_surface is not None:
        params["right-surface"] = right_surface
    if cerebellum_surface is not None:
        params["cerebellum-surface"] = cerebellum_surface
    return params


def cifti_false_correlation_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiFalseCorrelationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("3D-dist", None) is None:
        raise StyxValidationError("`3D-dist` must not be None")
    if not isinstance(params["3D-dist"], (float, int)):
        raise StyxValidationError(f'`3D-dist` has the wrong type: Received `{type(params.get("3D-dist", None))}` expected `float`')
    if params.get("geo-outer", None) is None:
        raise StyxValidationError("`geo-outer` must not be None")
    if not isinstance(params["geo-outer"], (float, int)):
        raise StyxValidationError(f'`geo-outer` has the wrong type: Received `{type(params.get("geo-outer", None))}` expected `float`')
    if params.get("geo-inner", None) is None:
        raise StyxValidationError("`geo-inner` must not be None")
    if not isinstance(params["geo-inner"], (float, int)):
        raise StyxValidationError(f'`geo-inner` has the wrong type: Received `{type(params.get("geo-inner", None))}` expected `float`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("left-surface", None) is not None:
        cifti_false_correlation_left_surface_validate(params["left-surface"])
    if params.get("right-surface", None) is not None:
        cifti_false_correlation_right_surface_validate(params["right-surface"])
    if params.get("cerebellum-surface", None) is not None:
        cifti_false_correlation_cerebellum_surface_validate(params["cerebellum-surface"])


def cifti_false_correlation_cargs(
    params: CiftiFalseCorrelationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-false-correlation"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    cargs.append(str(params.get("3D-dist", None)))
    cargs.append(str(params.get("geo-outer", None)))
    cargs.append(str(params.get("geo-inner", None)))
    cargs.append(params.get("cifti-out", None))
    if params.get("left-surface", None) is not None or params.get("right-surface", None) is not None or params.get("cerebellum-surface", None) is not None:
        cargs.extend([
            *(cifti_false_correlation_left_surface_cargs(params.get("left-surface", None), execution) if (params.get("left-surface", None) is not None) else []),
            *(cifti_false_correlation_right_surface_cargs(params.get("right-surface", None), execution) if (params.get("right-surface", None) is not None) else []),
            *(cifti_false_correlation_cerebellum_surface_cargs(params.get("cerebellum-surface", None), execution) if (params.get("cerebellum-surface", None) is not None) else [])
        ])
    return cargs


def cifti_false_correlation_outputs(
    params: CiftiFalseCorrelationParamsDict,
    execution: Execution,
) -> CiftiFalseCorrelationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiFalseCorrelationOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_false_correlation_execute(
    params: CiftiFalseCorrelationParamsDict,
    runner: Runner | None = None,
) -> CiftiFalseCorrelationOutputs:
    """
    COMPARE CORRELATION LOCALLY AND ACROSS/THROUGH SULCI/GYRI.
    
    For each vertex, compute the average correlation within a range of geodesic
    distances that don't cross a sulcus/gyrus, and the correlation to the
    closest vertex crossing a sulcus/gyrus. A vertex is considered to cross a
    sulcus/gyrus if the 3D distance is less than a third of the geodesic
    distance. The output file contains the ratio between these correlations, and
    some additional maps to help explain the ratio.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiFalseCorrelationOutputs`).
    """
    cifti_false_correlation_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_FALSE_CORRELATION_METADATA)
    params = execution.params(params)
    cargs = cifti_false_correlation_cargs(params, execution)
    ret = cifti_false_correlation_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_false_correlation(
    cifti_in: InputPathType,
    v_3_d_dist: float,
    geo_outer: float,
    geo_inner: float,
    cifti_out: str,
    left_surface: CiftiFalseCorrelationLeftSurfaceParamsDict | None = None,
    right_surface: CiftiFalseCorrelationRightSurfaceParamsDict | None = None,
    cerebellum_surface: CiftiFalseCorrelationCerebellumSurfaceParamsDict | None = None,
    runner: Runner | None = None,
) -> CiftiFalseCorrelationOutputs:
    """
    COMPARE CORRELATION LOCALLY AND ACROSS/THROUGH SULCI/GYRI.
    
    For each vertex, compute the average correlation within a range of geodesic
    distances that don't cross a sulcus/gyrus, and the correlation to the
    closest vertex crossing a sulcus/gyrus. A vertex is considered to cross a
    sulcus/gyrus if the 3D distance is less than a third of the geodesic
    distance. The output file contains the ratio between these correlations, and
    some additional maps to help explain the ratio.
    
    Args:
        cifti_in: the cifti file to use for correlation.
        v_3_d_dist: maximum 3D distance to check around each vertex.
        geo_outer: maximum geodesic distance to use for neighboring correlation.
        geo_inner: minimum geodesic distance to use for neighboring correlation.
        cifti_out: the output cifti dscalar file.
        left_surface: specify the left surface to use.
        right_surface: specify the right surface to use.
        cerebellum_surface: specify the cerebellum surface to use.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiFalseCorrelationOutputs`).
    """
    params = cifti_false_correlation_params(
        cifti_in=cifti_in,
        v_3_d_dist=v_3_d_dist,
        geo_outer=geo_outer,
        geo_inner=geo_inner,
        cifti_out=cifti_out,
        left_surface=left_surface,
        right_surface=right_surface,
        cerebellum_surface=cerebellum_surface,
    )
    return cifti_false_correlation_execute(params, runner)


__all__ = [
    "CIFTI_FALSE_CORRELATION_METADATA",
    "CiftiFalseCorrelationCerebellumSurfaceParamsDict",
    "CiftiFalseCorrelationCerebellumSurfaceParamsDictTagged",
    "CiftiFalseCorrelationLeftSurfaceParamsDict",
    "CiftiFalseCorrelationLeftSurfaceParamsDictTagged",
    "CiftiFalseCorrelationOutputs",
    "CiftiFalseCorrelationParamsDict",
    "CiftiFalseCorrelationParamsDictTagged",
    "CiftiFalseCorrelationRightSurfaceParamsDict",
    "CiftiFalseCorrelationRightSurfaceParamsDictTagged",
    "cifti_false_correlation",
    "cifti_false_correlation_cerebellum_surface",
    "cifti_false_correlation_execute",
    "cifti_false_correlation_left_surface",
    "cifti_false_correlation_params",
    "cifti_false_correlation_right_surface",
]
